#include "Configuration/CopterSoftwareConfig.h"
#include "appcode.h"
#include "objects.h"
#include "as/AS.h"
#include "app/componentheader.h"
#include "os/net/message.h"
#include "hw/dev/LAN91.h"
#include "hw/irq/DMA_SYSSRC2.h"
#include "os/net/message.h"
#include "os/net/arptable.h"
#include "os/net/ethernet.h"
#include "network_objects.h"
#include "TricoreHardware/gpta/GPTA.h"
#ifndef __puma
#include "TricoreHardware/adc/ADC.h"
#include "TricoreHardware/system/DMA.h"
#endif
#include <stdio.h>

#include "tc1796b/scu.h"
#include "tc1796b/ebu/addr.h"
#include "tc1796b/pmi/addr.h"

extern "C" {
/* initialization values for External Bus */
#define VAL_EBU_CON			0x0000FF68
#define VAL_EBU_BFCON		0x001001D0
/* CS0: 1 MB external MRAM */
#define VAL_EBU_ADDRSEL0	0xA1000873
#define VAL_EBU_BUSCON0		0x00920000
#define VAL_EBU_BUSAP0		0x81200010
/* CS1: 4 MB external Flash (not used) */
#define VAL_EBU_ADDRSEL1	0xA4000853
#define VAL_EBU_BUSCON1		0x00920000
#define VAL_EBU_BUSAP1		0xC7B30000
/* CS2 is used for USB controller */
#define VAL_EBU_ADDRSEL2	0xD90000C1
#define VAL_EBU_BUSCON2		0x00420000
#define VAL_EBU_BUSAP2		0x40D809F0
/* CS3 is used for LAN controller */
#define VAL_EBU_ADDRSEL3	0xD80000C1
#define VAL_EBU_BUSCON3		0x00820000
#define VAL_EBU_BUSAP3		0x40D80000

/* enable instruction cache */
#define VAL_PMI_CON0		0x00000000

	typedef struct
	{
		unsigned long addr;
		unsigned long val;
	} InitTab_t;

	const InitTab_t boardSetupTab[] =
	{
		{EBU_ADDRSEL0_ADDR,	VAL_EBU_ADDRSEL0},
		{EBU_BUSCON0_ADDR,	VAL_EBU_BUSCON0	},
		{EBU_BUSAP0_ADDR,	VAL_EBU_BUSAP0	},
		{EBU_ADDRSEL1_ADDR,	VAL_EBU_ADDRSEL1},
		{EBU_BUSCON1_ADDR,	VAL_EBU_BUSCON1	},
		{EBU_BUSAP1_ADDR,	VAL_EBU_BUSAP1	},
		{EBU_ADDRSEL2_ADDR,	VAL_EBU_ADDRSEL2},
		{EBU_BUSCON2_ADDR,	VAL_EBU_BUSCON2	},
		{EBU_BUSAP2_ADDR,	VAL_EBU_BUSAP2	},
		{EBU_ADDRSEL3_ADDR,	VAL_EBU_ADDRSEL3},
		{EBU_BUSCON3_ADDR,	VAL_EBU_BUSCON3	},
		{EBU_BUSAP3_ADDR,	VAL_EBU_BUSAP3	},
		/*
			This must be done after CSx programming to avoid disabling EBU_CON.CS0FAM
			(CS0 Fills Address Map)
		 */
		{EBU_CON_ADDR,		VAL_EBU_CON		},
		{EBU_BFCON_ADDR,	VAL_EBU_BFCON	},
	};
	
	const unsigned long boardSetupTabSize = sizeof(boardSetupTab) / sizeof(InitTab_t);

void __board_init (void) __attribute__((interrupt));
void __board_init (void) {
	InitTab_t* lauf = (InitTab_t*)boardSetupTab;
	while ( lauf != boardSetupTab +  boardSetupTabSize ) {
		*((uint32_t *)lauf->addr) = lauf->val;
		lauf++;
	}
}

void run_app_hw_init(){
    SystemFrequency::Instance().init();

    /* Initialize Tricore Hardware */
    GPIO::init();
    GPTA::init(1250000);
    GPTA::initModule(0);
    GPTA::initModule(1);

#ifndef __puma
    ADCSystem::init(12500000);
    ADC<0>::init();
    ADC<1>::init();

    #if SPI_BUS == 1
    SSC<0>::init(12500000);
    SSC<0>::setupPorts(SSC_MODE_MASTER);
    SSC<0>::setupConfiguration(8, SSC_HEADING_MSB, SSC_POLARITY_LOW, SSC_PHASE_RECEIVE);
    SSC<0>::setBaudrate(75000);
    SSC<0>::setupSlaveTiming(2, 0, 0);
    SSC<0>::enableFifo();
    SSC<0>::enableErrorChecking();
    SSC<0>::setRXFifoLevel(8);
    SSC<0>::enableInterrupts(0, INT_SPI, INT_SPI_ERR);
    #endif

    #if RS232_BUS >= 1
    RS232<ASC0_ID>::init(RS232_ASC0_BAUDRATE);
    #endif
    #if RS232_BUS == 2
    RS232<ASC1_ID>::init(RS232_ASC1_BAUDRATE, false);
    RS232<ASC1_ID>::enableErrorDetection();
    ASC1_TSRC.bits.SRPN=INT_RS232_RADIO;
    ASC1_TSRC.bits.SRE=1;
    #endif

    #if DMA_RADIO == 1
    DMA<ASC1_ID>::setup(0xF010A000,  0x00000C0, 4); // 2^4 = 16 bytes
    DMA<ASC1_ID>::config_channel_irq(INT_DMA_RADIO);
    DMA<ASC1_ID>::asc_start();
    DMA<ASC1_ID>::enable_irq();
    #endif

    #if NAVITASK == 1
    /* DMA Initialisation */
    DMA<ASC0_ID>::setup(0xF010A080,  0x000000C0, 7); // 2^7 = 128 byte
    DMA<ASC0_ID>::config_pattern_irq('\r', '\n', INT_DMA_GPS);
    DMA<ASC0_ID>::asc_start();
    DMA<ASC0_ID>::enable_irq();
    #endif
#endif
}
}

TASKDEF(InitTaskComponent, InitTask) {
	AS::SetRelAlarm(wakeCopterControl, 7,7);          // 21ms period
	AS::SetRelAlarm(wakeCopterControlTelemetry, 9,9); // 27 ms period
#ifdef cfcopter_flightcontrol_c
	AS::SetRelAlarm(wakeFlightControl, 3,3);          // 9ms  period
#endif
#ifdef cfcopter_flightcontrol_java
	AS::SetRelAlarm(wakeJFlightControl, 3,3);         // 9ms  period
#endif
	AS::SetRelAlarm(wakeSignalProcessing, 1, 1);      // 3ms  period
	AS::ActivateTask(SPIComTask);
	AS::ActivateTask(EthernetTask);

	AS::TerminateTask();
}

TASKDEF(InitTaskComponent, restartDMATask) {
#ifndef __puma
	DMA<ASC1_ID>::asc_start();
#endif

	AS::TerminateTask();
}

TASKDEF(IPStackComponent, IPStackTask) {
	uint16_t len; 
	hw::dev::LAN91::NicInit();
	os::net::Message msg;

// 	AS::ActivateTask(InitTask);

	/* configure P1.0 for use as external interrupt pin REQ0 */
	EICR0.bits.EXIS0 = 0;	/* input line IN00 */
	EICR0.bits.REN0  = 1;	/* rising edge */
	EICR0.bits.LDEN0 = 1;	/* automatic reset by falling edge */
	EICR0.bits.EIEN0 = 1;	/* external interrupt enable */
	EICR0.bits.INP0  = 0;	/* interrupt node pointer INT00 */

	hw::irq::DMA_SYSSRC2::enable();

	os::net::Arptable::Init();

	SocketBuffer* toSend;

	while ( 1 ) {
		len = hw::dev::LAN91::NicGetPacket(msg.data, sizeof(msg.data));
		if ( len > 0 ) {
			msg.size = len;
			os::net::Ethernet::Input(msg);
		}

		toSend = EthernetTaskIPStack_receiver.receiveMessage();
		if ( toSend != 0x0 ) {
			os::net::Message packet;

			packet.size = toSend->size;
			packet.udp.header.uh_dport = toSend->sport;
			memcpy(packet.udp.data, toSend->buffer, toSend->size);

			os::net::Udp::Output(toSend->dip, toSend->dport, packet);
			EthernetTaskIPStack_receiver.releaseMessage(toSend);
		}

		AS::WaitEvent(NetworkHardwareEvent | PacketWaitingEvent);
		AS::ClearEvent(PacketWaitingEvent | NetworkHardwareEvent);
	}

	AS::TerminateTask();
}

void IPStackComponent::functionISRNetworkISR() {
    AS::SetEvent(IPStackTask, NetworkHardwareEvent);
}

