#!/usr/bin/perl

use Getopt::Long;

use strict;
use warnings;

unless (-d "skel") {
	print STDERR "application template directory \"skel\" does not exist\n";
	exit 1;
}

my $variant = 'default';

my $result = GetOptions ( "variant=s" => \$variant );
unless( $result and @ARGV == 1 and -d "skel/.$variant" ) {
	print STDERR "Usage: $0 [--variant <variant>] <appname>\n";
	print STDERR "Available variants are:\n";
	opendir(DIR, "skel") or die "failed to opendir \"skel\": $!\n";
	my @variants = grep(/^\./, readdir(DIR));
	closedir(DIR);

	@variants = grep(-d "skel/$_", @variants);
	@variants = grep(-f "skel/$_-desc", @variants);

	foreach my $v (@variants) {
		open(DESC, '<', "skel/$v-desc") or die "could not open description for variant $v";
		$v =~ s/^\.//;
		print STDERR " * $v\n";
		while(<DESC>) {
			print STDERR "\t$_";
		}
		close(DESC);
	}
	exit 1;
}


my $app = $ARGV[0];

if(-d $app) {
	print STDERR "Application directory $app exists!\n";
	exit 1;
}

mkdir $app or die "could not application directory $app: $!\n";
copyDir("skel", $app);
copyDir("skel/.$variant", $app);

print <<EODONE;
Template directory for application $app successfully created.

=======================      Optional steps to do      =======================

- create an application-specific feature model for applications with
  configurable features by editing
	$app/$app.fm

- define the actions for your application specific features by editing
	$app/$app.cmp.pl

=================      Proceed with the following steps      =================

- configure the OS level objects by editing
	$app/$app.profile.pl

- configure the OS features and OS object attributes
	cd $app && make xconfig

- create the configured OS variant (default output to $ENV{'CIAOBASE'}/kconf/config/app/$app)
	cd $app && make transform

- compile the variant
	cd $ENV{'CIAOBASE'}/kconf/config/app/$app && make
EODONE

#############################################################################
sub copyDir {
	my ($src,$dst) = @_;
	opendir(DIR, $src) or die "could not opendir $src: $!\n";
	my @ent = readdir(DIR);
	closedir(DIR);
	@ent = grep(!/^\./, @ent);

	foreach my $ent (@ent) {
		my $sf = "$src/$ent";
		my $df = $ent;
		$df =~ s/^skel\./$app./;
		$df = "$dst/$df";

		if ( -d $sf ) {
			mkdir $df or die "could not create directory $df: $!\n";
			copyDir($sf,$df);
		} elsif ( -f $sf ) {
			open(IN,  '<', $sf) or die "could not open file $sf for reading: $!\n";
			open(OUT, '>', $df) or die "could not open file $df for writing: $!\n";
			while(<IN>) {
				s/###SKEL###/$app/g;
				print OUT;
			}
			close(IN);
			close(OUT);
		}
	}
}

