#ifndef _GNU_SOURCE
#define _GNU_SOURCE
#endif

#include <stdlib.h>
#include <stdio.h>
#include <errno.h>

#include "control.h"

#include "generator_errors.h"

program_t* program_definition();

int main(int argc, char** argv) {
	FILE* fragFile;
	FILE* targetFile;
	size_t size = 100;
	char* buffer;
	program_t* prog;
	config_t* conf;
	config_elem_t* iter;
	
	if(argc != 3) return FILE_ERROR;
	
	fragFile = fopen(argv[1],"r");
	if(fragFile == NULL) {
		perror(argv[1]);
		return FILE_ERROR;
	}
	
	targetFile = fopen(argv[2],"w");
	if(targetFile == NULL) {
		perror(argv[2]);
		return FILE_ERROR;
	}
	
	buffer = (char*) malloc(size);
	if(buffer == NULL) return NOMEM_ERROR;
	
	errno = 0;
	while(true) {
		int ret = getline(&buffer, &size, fragFile);
		if(ret == EOF) {
			if(errno != 0) {	
				perror("Error");
				return FILE_ERROR;
			}
			break;
		}
		fprintf(targetFile, buffer);
	}
	
	/* Program is generated by linking with a function that calls program functions */
	prog = program_definition();
	
	conf = CTRL_Compile(prog);
	fprintf(targetFile, "static command commands[] = {\n");
	
	if(conf == NULL) {
		fprintf(stderr, "Error at command: %d\n", prog->errorPos);
		fclose(targetFile);
		remove(argv[2]);
		
		return COMPILER_ERROR;
	}
	
	
	for(iter = conf->firstElem; iter != NULL; iter = iter->next) {
		switch(iter->cmd.mtype) {
			case MOVE_UP:
				fprintf(targetFile, "\t{MOVE_UP,%d}", iter->cmd.n);
				break;
			case MOVE_DOWN:
				fprintf(targetFile, "\t{MOVE_DOWN,%d}", iter->cmd.n);
				break;
			case MOVE_HOLD:
				fprintf(targetFile, "\t{MOVE_HOLD,%d}", iter->cmd.n);
				break;
			case MOVE_RELEASE:
				fprintf(targetFile, "\t{MOVE_RELEASE,%d}", iter->cmd.n);
				break;
			case MOVE_END:
				fprintf(targetFile, "\t{MOVE_END,%d}", 0);
				break;
			case MOVE_START:
				/* Ignore this */
				continue;
			default:
				break;
		}
		
		if(iter->next != NULL) {
			fprintf(targetFile, ",\n");
		} else {
			fprintf(targetFile, "\n");
		}
	}
	
	fprintf(targetFile, "\n};\n\n");
	fclose(targetFile);
	
	return 0;
}
