# Architecture
#GCC_ARCH ?= i686-pc-linux-gnu
# GCC from Debian 5.0 (Lenny)
GCC_ARCH ?= i486-linux-gnu

# Defaults
CXX     ?= g++
CC      ?= gcc
LD      := g++
NASM    ?= nasm
OBJCOPY ?= objcopy

CXX_PATH  ?= /usr/bin
ACXX_PATH ?= /opt/ciao_tools/bin
AGXX_PATH ?= /opt/ciao_tools/bin

include $(MODULES_DIR)/ciao_config_arch.mk
#CXXFLAGS  += -m32
#NASMFLAGS += -f elf32

# link against libgcc for 64-bit div
LIBS     += gcc

# Function-Level-Linking
CXXFLAGS += -ffunction-sections -fno-use-cxa-atexit -fno-rtti -fno-exceptions -Wno-write-strings
ASFLAGS  += -ffunction-sections
LDFLAGS  += --gc-sections

# Workaround, da Stephan die Option "-nostdlib" nicht mag
#CIAO_LINKFIRST += -nostdlib

# flags for ag++
#AGXXFLAGS += -v9

# we link with g++, therefore transform LDFLAGS
LINKFLAGS := $(foreach opt,$(LDFLAGS),-Xlinker $(opt)) $(CXXFLAGS)

# Compile a NASM-Assembler File in the '$(SRC_DIR)' directory
#$(OUT_DIR)/%.o: $(SRC_DIR)/%.asm
#	@echo "(NASM)    $(call printpath,$@)"
#	@if [ ! -e $(@D) ];then mkdir -p $(@D);fi
#	@$(NASM) -g $(NASMFLAGS) -O100 $< -o $@

# how to link a elf binary
$(APP).elf : all $(OBJ) $(CIAO_LIBS:%=$(LIB_DIR)/lib%.a)
	@echo $(OBJ) $(CIAO_LIBS:%=$(LIB_DIR)/lib%.a)
	@echo "(LINK)    $(APP).elf"
	$(LD) $(LINKFLAGS) -o $(OUT_DIR)/$(APP).elf $(CIAO_LINKFIRST) $(OBJ) -L$(LIB_DIR) $(CIAO_LIBS:%=-l%) $(LIBS:%=-l%)

test_link:
	@echo $(LD) $(LINKFLAGS) -o $(OUT_DIR)/$(APP).elf $(CIAO_LINKFIRST) $(OBJ) -L$(LIB_DIR) $(CIAO_LIBS:%=-l%) $(LIBS:%=-l%)

# create plain binary executable
$(APP) : $(APP).elf
	@echo "(OBJCOPY) $@"
	@if test \( ! \( -d $(@D) \) \) ;then mkdir -p $(@D);fi
	$(OBJCOPY) -O binary $< $@

# create a Floppy-Image
$(APP).img: $(APP) $(LIB_DIR)/bootsect.bin16 $(LIB_DIR)/setup.bin16
	@dd if=/dev/zero of=$(APP).img bs=512 count=2880
	@dd if=$(LIB_DIR)/bootsect.bin16 of=$(APP).img conv=notrunc bs=512
	@dd if=$(LIB_DIR)/setup.bin16 of=$(APP).img seek=1 conv=notrunc bs=512
	@dd if=$(APP) of=$(APP).img seek=2 conv=notrunc bs=512

# 80 tracks
	@echo -e -n '\0120\0000' | dd of=$(APP).img bs=1 seek=4 conv=notrunc
# 2 heads
	@echo -e -n '\0002\0000' | dd of=$(APP).img bs=1 seek=6 conv=notrunc
# 18 sectors
	@echo -e -n '\0022\0000' | dd of=$(APP).img bs=1 seek=8 conv=notrunc
# Setup-Code-Größe (in 512 Byte Blöcken)
# TODO: Größe dynamisch bestimmen
	@echo -e -n '\0001\0000' | dd of=$(APP).img bs=1 seek=10 conv=notrunc
# System-Code-Größe (in 512 Byte Blöcken)
# TODO: Größe dynamisch bestimmen
	@echo -e -n '\0037\0000' | dd of=$(APP).img bs=1 seek=12 conv=notrunc
# Bootdevice Code: 00: Disk A, 01: Disk B, ..., 0x80 HD0, ...
	@echo -e -n '\0000' | dd of=$(APP).img bs=1 seek=14 conv=notrunc
# Start-Track
	@echo -e -n '\0000' | dd of=$(APP).img bs=1 seek=15 conv=notrunc
# Start-Head
	@echo -e -n '\0000' | dd of=$(APP).img bs=1 seek=16 conv=notrunc
# Start-Sector
	@echo -e -n '\0001' | dd of=$(APP).img bs=1 seek=17 conv=notrunc

$(APP).elf.hd: $(APP).elf
	@echo "Creating 40 MiB Image"
	@dd if=/dev/zero bs=1M count=0 seek=40 of=$(APP).elf.hd
	@dd if=$(APP).elf of=$(APP).elf.hd conv=notrunc

$(APP)-module.img:
	@echo "Creating RomFS-Image"
	@genromfs -d romfs -f $(APP)-module.img

$(APP).elf.hd-module: $(APP).elf.hd $(APP)-module.img
	@echo "Insert Multiboot Module at block 8192"
	@dd if=$(APP)-module.img of=$(APP).elf.hd seek=8192 conv=notrunc

qemu: $(APP).elf
	@echo "(QEMU)    $(APP)"
	qemu-system-x86_64 -m 128 -kernel $(APP).elf

qemu-hd: $(APP).elf
	@echo "(QEMU)    $(APP)"
	qemu-system-x86_64 -m 128 -kernel $(APP).elf -hda $(HOME)/testdisk.img

kqemu: $(APP).elf
	@echo "(QEMU)    $(APP)"
	@qemu -m 16 -kernel $(APP).elf \
	      -net nic,model=e1000,macaddr=00:11:22:33:44:55 -net vde,sock=/var/run/vde2/tap0.ctl

kvm: $(APP).elf
	@echo "(KVM)     $(APP)"
	kvm -m 128 -kernel $(APP).elf

kvm-hd: $(APP).elf
	@echo "(KVM)     $(APP)"
	kvm -m 128 -kernel $(APP).elf -hda $(HOME)/testdisk.img

size: $(APP).elf
	@echo "(SIZE)    $(APP)"
	@size $(APP).elf

qemu-grub: $(APP).elf.hd
	@echo "(QEMU)    $(APP)"
	@qemu -m 128 -snapshot -fda $(LIB_DIR)/grub-floppy.img -hda $(APP).elf.hd -boot a

kvm-grub: $(APP)
	@echo "(QEMU)    $(APP)"
	@kvm -m 128 -snapshot -fda $(LIB_DIR)/grub-floppy.img -hda $(APP).elf.hd -boot a
	
qemu-grub2: $(APP).elf.hd-module
	@echo "(QEMU)    $(APP)"
	@qemu-system-x86_64 -m 128 -d in_asm,int,exec,cpu,pcall,cpu_reset -no-reboot -fda $(LIB_DIR)/grub2-floppy.img -boot a -hda $(APP).elf.hd -net nic,model=e1000 -net socket,mcast=224.10.0.1:9999

sysqemu-grub2: $(APP).elf.hd-module
	@echo "(QEMU)    $(APP)"
	/usr/bin/kvm -m 128 -no-reboot -fda $(LIB_DIR)/grub2-floppy.img -boot a -hda $(APP).elf.hd -net nic,model=e1000 -net socket,mcast=224.10.0.1:9999

start-sysqemu-grub2:
	@echo "(QEMU)    $(APP)"
	/usr/bin/kvm -m 128 -no-reboot -fda $(LIB_DIR)/grub2-floppy.img -boot a -hda $(APP).elf.hd -net nic,model=e1000 -net socket,mcast=224.10.0.1:9999

grub2-image: $(APP).elf
	@echo "(MKIMAGE) $(APP).grub2"
	@grub-mkimage -o $(APP).grub2 -m $(APP).elf -c $(CIAO_DIR)/hw/grub2.cfg \
	acpi biosdisk blocklist boot cpuid configfile halt help hexdump lspci memdisk minicmd msdospart multiboot pci reboot serial sh vga vbeinfo videotest video
	
grub2-image-strip: $(APP).elf
	@echo "(MKIMAGE) $(APP).grub2"
	@strip -s $(APP).elf
	@dd if=/dev/zero of=$(APP).elf bs=512 count=0 seek=256
	@grub-mkimage -o $(APP).grub2 -m $(APP).elf -c $(CIAO_DIR)/hw/grub2.cfg \
	acpi biosdisk blocklist boot cpuid configfile halt help hexdump lspci memdisk minicmd msdospart multiboot pci reboot serial sh vga vbeinfo videotest video

grub2-floppy: grub2-image
	@echo "(DD)      $(APP).grub2-floppy"
	@cat `grub-mkimage --help | grep -- '--directory' | sed 's/^.*default=\(.*\)\]/\1/'`/boot.img $(APP).grub2 > $(APP).grub2-floppy
	@dd if=/dev/zero of=$(APP).grub2-floppy bs=512 count=0 seek=2880 2>/dev/null

grub2-floppy-strip: grub2-image-strip
	@echo "(DD)      $(APP).grub2-floppy"
	@cat `grub-mkimage --help | grep -- '--directory' | sed 's/^.*default=\(.*\)\]/\1/'`/boot.img $(APP).grub2 > $(APP).grub2-floppy
	@dd if=/dev/zero of=$(APP).grub2-floppy bs=512 count=0 seek=2880 2>/dev/null

qemu-grub2-floppy: grub2-floppy
	@echo "(QEMU)    $(APP)"
	qemu-system-x86_64 -m 128 -d in_asm,int,exec,cpu,pcall,cpu_reset -no-reboot \
                           -fda $(APP).grub2-floppy -boot a \
                           -net nic,model=e1000,macaddr=00:11:22:33:44:55 -net vde,sock=/tmp/bossel-vde
                           
syskvm-grub2-floppy-strip: grub2-floppy-strip
	@echo "(QEMU)    $(APP)"
	kvm -m 128 -no-reboot \
                           -fda $(APP).grub2-floppy -boot a \
                           -net nic,model=e1000,macaddr=00:11:22:33:44:55 -net user

qemu-grub2-floppy-hd: grub2-floppy
	@echo "(QEMU)    $(APP)"
	qemu-system-x86_64 -m 128 -d in_asm,int,exec,cpu,pcall,cpu_reset -no-reboot \
                           -fda $(APP).grub2-floppy -boot a \
                           -net nic,model=e1000,macaddr=00:11:22:33:44:55 -net vde,sock=/tmp/bossel-vde \
                           -hda $(HOME)/testdisk.img

qemu-grub2-floppy-dump: grub2-floppy
	@echo "(QEMU)    $(APP)"
	qemu-system-x86_64 -m 128 -d in_asm,int,exec,cpu,pcall,cpu_reset -no-reboot \
                           -fda $(APP).grub2-floppy -boot a \
                           -net nic,model=e1000,macaddr=00:11:22:33:44:55 -net dump,file=/tmp/bossel-netdump

qemu-virtio-grub2-floppy: grub2-floppy
	@echo "(QEMU)    $(APP)"
	qemu-system-x86_64 -m 128 -enable-kvm -no-reboot \
                           -fda $(APP).grub2-floppy -boot a \
                           -net nic,model=virtio,macaddr=00:11:22:33:44:55 -net vde,sock=/tmp/bossel-vde
