#
# MODULE: generate
#
# generate CiAO runtime from configuration

OUT_DIR ?= $(LIB_DIR)

XSLT_DIR ?= $(ROOT_DIR)/generator
PERL_SCRIPT_DIR ?= $(ROOT_DIR)/generator
APP_PERL_SCRIPT_DIR ?= $(OUT_DIR)/ciao_generate

# find generator transformers
XSLT ?= $(shell find $(XSLT_DIR) -follow -name '*.xsl' )

# OS PERLSCRIPTS are scripts that generate a file located within the tree
# of the operating system, because the generated file is required to compile
# the OS itself
#
# These generator scripts must have the prefix _os_ in their name and an
# all-capital PL suffix
OSPERLSCRIPTS ?= $(shell find $(PERL_SCRIPT_DIR) -follow -name '_os_*.PL' )

# PERLSCRIPTS are scripts that generate a file located in the application
# source directory. These are only needed to compile the application but not
# for the OS itself
PERLSCRIPTS ?= $(shell find $(PERL_SCRIPT_DIR) -follow -name '*.pl' )

# APP PERLSCRIPTS are generator scripts _provided by_ the application, that
# generate files only needed for that specific application
#
# These scripts are located within a subdirectory of the application source
# directory, which defaults to ciao_generate
APPPERLSCRIPTS ?= $(shell [ -d $(APP_PERL_SCRIPT_DIR) ] && find $(APP_PERL_SCRIPT_DIR) -follow -name '*.pl' )

# configuration files
#CFG  ?= $(SRC_DIR)/config.xml $(ROOT_DIR)/variant.xml
CFG  ?= $(SRC_DIR)/config.xml
CIAO_CONFIG ?= $(APP).config

# if ComStacks.xml.xsl exists, we must add ComStacks.xml to the list
# of Config files. Additionally we need a node.h in this case as dependency
CFG += $(if $(filter %/ComStacks.xml.xsl, $(XSLT)), $(SRC_DIR)/ComStacks.xml)
COM_DEP ?= $(if $(filter %/ComStacks.xml.xsl, $(XSLT)), $(SRC_DIR)/node.h)


# generated source files
GEN_SRC  =  $(patsubst $(XSLT_DIR)/%.xsl,$(OUT_DIR)/%,$(XSLT))
GEN_SRC  += $(patsubst $(PERL_SCRIPT_DIR)/_os_%.PL,$(ROOT_DIR)/%,$(OSPERLSCRIPTS))
GEN_SRC  += $(patsubst $(PERL_SCRIPT_DIR)/%.pl,$(OUT_DIR)/%,$(PERLSCRIPTS))
GEN_SRC  += $(patsubst $(APP_PERL_SCRIPT_DIR)/%.pl,$(OUT_DIR)/%,$(APPPERLSCRIPTS))

# resulting dependencies (object files, linker scripts) to link application
GEN  = $(filter %.o %.x, $(patsubst %.cpp, %.o, $(GEN_SRC)))

# add clean 
MODULES_CLEAN += generate

# add all 
MODULES_ALL += generate

# 'clean' target for 'compile' MODULE
clean_generate: 
	@rm -f $(GEN_SRC)

# 'all' target for 'compile' MODULE
all_generate: $(GEN_SRC)

#Generate a File from xslt- and xml-files
$(OUT_DIR)/%: $(XSLT_DIR)/%.xsl $(CFG) $(COM_DEP)
	@echo "(GEN)    $@"
	@if [ ! -e $(@D) ];then mkdir -p $(@D);fi 
	@xsltproc $< $(CFG) > $@ 

$(ROOT_DIR)/%: $(PERL_SCRIPT_DIR)/_os_%.PL $(CFG) $(COM_DEP) $(CIAO_CONFIG)
	@echo "(OS PLGEN)    $@"
	@if [ ! -e $(@D) ];then mkdir -p $(@D);fi 
	@perl -I$(PERL_SCRIPT_DIR) $< $(CFG) $(CIAO_CONFIG) $@

# Generate a File from pl- and xml-files
$(OUT_DIR)/%: $(PERL_SCRIPT_DIR)/%.pl $(CFG) $(COM_DEP) $(CIAO_CONFIG)
	@echo "(PLGEN)    $@"
	@if [ ! -e $(@D) ];then mkdir -p $(@D);fi 
	@perl -I$(PERL_SCRIPT_DIR) $< $(CFG) $(CIAO_CONFIG) $@

# Generate a File from pl- and xml-files
$(OUT_DIR)/%: $(APP_PERL_SCRIPT_DIR)/%.pl $(CFG) $(COM_DEP) $(CIAO_CONFIG)
	@echo "(PLGEN)    $@"
	@if [ ! -e $(@D) ];then mkdir -p $(@D);fi 
	@perl -I$(PERL_SCRIPT_DIR) $< $(CFG) $(CIAO_CONFIG) $@


#Specials for Communication Infrastructure
$(SRC_DIR)/ComInfrastructureIntermediate.xml: $(XSLT_DIR)/ComInfrastructureIntermediate.xml.xsl $(SRC_DIR)/ComInfrastructure.xml
	@echo "(GEN)    $@"
	@if [ ! -e $(@D) ];then mkdir -p $(@D);fi 
	@xsltproc $< $(SRC_DIR)/ComInfrastructure.xml > $@ 

$(SRC_DIR)/ComStacks.xml: $(XSLT_DIR)/ComStacks.xml.xsl $(SRC_DIR)/ComInfrastructureIntermediate.xml
	@echo "(GEN)    $@"
	@if [ ! -e $(@D) ];then mkdir -p $(@D);fi 
	@xsltproc $< $(OUT_DIR)/ComInfrastructureIntermediate.xml > $@ 
