/*
 * GDT.cpp
 *
 *  Created on: Feb 23, 2010
 *      Author: bossel00
 */

#include "GDT.h"
#include "GDT_flags.h"
#include "GDT_TSS_flags.h"

// TODO: Anzahl Tasks herausbekommen?
#include "ciaoConfig.h"
#define NUMBER_OF_TASKS cfOS_NUMBER_OF_TASKS

namespace hw {

GlobalDescriptorTable gdt;

GDT GDT::inst_;
const int GDT::ENTRIES = cfX86_GDT_ENTRIES;
int GDT::last_entry = 3 + X86_TSS;
Descriptor GDT::descriptors[GDT::ENTRIES];

void GDT::init() {
	descriptors[0].value = 0;                                     // Null-Descriptor
	descriptors[1].value = X86_CODE_SEGMENT_DESCRIPTOR_VALUE;     // Code-Segment-Descriptor
	descriptors[2].value = 0x00CF92000000FFFFULL;                 // Data-Segment-Descriptor

	if (X86_TSS)
		descriptors[3].value = 0x0000850000000000ULL;            // Task-Gate-Descriptor (TODO: in before-advice auslagern)

	gdt.limit = cfX86_GDT_ENTRIES * 8 - 1;
	gdt.base = descriptors;

	// setup new GDT
	asm volatile("lgdt %0" : : "m"(gdt) : "memory");

	// setup data segments
	asm volatile("movl %0, %%ds;"
			   "movl %0, %%gs;"
			   "movl %0, %%es;"
			   "movl %0, %%fs;"
			   "movl %0, %%ss;"
			  : : "r"(0x10) : "memory");

	// setup code segment
	/* Geht nur im 32-Bit Modus
	asm volatile("jmp %0, $go$;"
			   "go$:"
			   : : "i"(0x8) : "memory");
	*/

	// setup code segment (lret hack)
	/* Geht auch nicht... GPF :-(
	asm volatile("sub $10, %%rsp;"
			   "mov $go$, %%rax;"
			   "movq %%rax, 0(%%rsp);"
			   "movw %0, 8(%%rsp);"
			   "lret;"
			   "go$:"
			   : : "i"(0x8) : "memory", "%rax");
	*/

	// setup code segment (absolute indirect 64-Bit far jump)
	/*  Geht genauso wenig... GPF :-(
	asm volatile("sub $10, %%rsp;"
			   "mov $go$, %%rax;"
			   "movq %%rax, 0(%%rsp);"
			   "movw %0, 8(%%rsp);"
			   "ljmpq *0(%%rsp);"        // problem: no m16:64 operand?
			   "go$:"
			   "add $10, %%rsp;"
			   : : "i"(0x8) : "memory", "%rax");
	 */

	// load new code segment for 32-bit in GDT_code32.ah
	// TODO: load code segment for 64-bit?
}

int GDT::getDescriptor() {
	return ++last_entry;
}

Descriptor* GDT::getDescriptor(int index) {
	return &descriptors[index];
}

Descriptor& GDT::operator[](int index) {
	return *getDescriptor(index);
}

}
