// $Id: AVRI2CChannel.h 995 2009-08-11 14:26:31Z mike $

#ifndef __hw_dev_AVRI2CChannel__h__
#define __hw_dev_AVRI2CChannel__h__

/**
 * \file AVRI2CChannel.h
 */

#include <avr/io.h> // AVR i/o port definitions for avr-libc

/*! \class AVRI2CChannel
 * \brief AVR I2C access channel
 *
 * This class is needed for the implementation of the I2C bus protocol
 * on the pure-systems AVR board. It implements IC on pin 2 (clock) and 3 (data) of PORTE.
 */

namespace hw { namespace dev {}
   
	class AVRI2CChannel {
	
	    enum { SCLVAL = 0x04, SDAVAL = 0x08 };
	    
	public:
	    //! sets the SDA line to LOW
	    void sda_lo () { DDRE |= SDAVAL; }
	    
	    //! sets the SDA line to HIGH
	    void sda_hi () { DDRE &= ~SDAVAL; }
	    
	    //! sets the SCL line to LOW
	    void scl_lo () { DDRE |= SCLVAL; }
	    
	    //! set the SCL line to HIGH and waits until the line comes up
	    void scl_hi () {
			DDRE &= ~SCLVAL;
    		while( !get_scl() );
		}
	    
	    //! read the SDA line
	    bool get_sda () { return (PINE & SDAVAL) != 0; }
	    
	    //! read the SCL line
	    bool get_scl () { return (PINE & SCLVAL) != 0; }
	    
	    //! wait some time
	    void delay () {
	        asm("nop\nnop\nnop\nnop");
	    }
	    
	};
} } // namespace hw::dev

#endif // __hw_dev_AVRI2CChannel__h__
