// $Id: P1.h 995 2009-08-11 14:26:31Z mike $
// Created:   05.06.2006 lohmann

/**
 * @file P1.h
 * @brief Contains abstractions for the TC1796 General Purpose IO (GPIO) lines
 */
 
#ifndef HW_DEV_P1_H_
#define HW_DEV_P1_H_

#include "hw/dev/GPIO.h"

namespace hw { namespace dev {

/**
 * @brief TC1796 GPIO Port 1
 * 
 * Port 1 is a 16-bit bi-directional general-purpose I/O port that can be alternatively used for
 * the MLI0 I/O lines or for the external trigger inputs REQ0-REQ3 of the CPU. Furthermore,
 * the system clock output SYSCLK is provided at Port 1.12
 * 
 * @note P1.0-P1.3 are connected to the green LEDs of our experimental board. P1.15 is connected to the /a
 * toggle LED of the TriBoard1796.
 * 
 */
class P1 : public PortBase< 1, 16, 0xffff > {
	static P1 theP1_;
	//! Performs start-time initializations. Only called by advice.
	void init() {
		unsigned long* p = &port().IOCR[0].reg;
		*p++ = cfDEV_P1_IOCR0;
		*p++ = cfDEV_P1_IOCR4;
		*p++ = cfDEV_P1_IOCR8;
		*p   = cfDEV_P1_IOCR12;
	}
  protected:
    P1(){}
  public:	
	//! Returns the one-and-only P1 instance
	static P1& Inst() {
		return theP1_;
	}
};



}} // namespace hw::dev
#endif /*HW_DEV_P1_H_*/
