#ifndef __HW_DEV_PIT_H__
#define __HW_DEV_PIT_H__

#include "stdinc.h"
#include "IOPort.h"
#include "ciaoConfig.h"

namespace hw {
namespace dev {

class PIT {

private:

	struct control_byte {
		UInt8 counter :2;
		UInt8 rw :2;
		UInt8 mode :3;
		UInt8 format :1;

		operator UInt8() {
			return (counter << 6) | (rw << 4) | (mode << 1) | (format << 0);
		}
	};

	unsigned us_per_tick;

	static PIT thePIT_;

	IOPort counter_[3];
	IOPort control_;

public:
	void init() {
		counter_[0].setAddress(cfDEV_PIT_C0_ADDRESS);
		counter_[1].setAddress(cfDEV_PIT_C1_ADDRESS);
		counter_[2].setAddress(cfDEV_PIT_C2_ADDRESS);
		control_.setAddress(cfDEV_PIT_CTRL_ADDRESS);

		setInterval(1000);  // 1 ms
	}

	static PIT& Inst() {
		return thePIT_;
	}

	int getInterval();

	void setIntervalTicks(UInt16 ticks);
	void setInterval(int us);

	void set_speaker(unsigned short freq);
	void speaker_off();

	unsigned get_us_per_tick() {
		return us_per_tick;
	}

	static const int FREQ = 1000;

};

}
}

#endif // __HW_DEV_PIT_H__
