/** \author Christian Meier
 *  \version $Id: Controller.h 995 2009-08-11 14:26:31Z mike $
 */

#ifndef HW_DEV_CAN_MULTICAN_CONTROLLER_H
#define HW_DEV_CAN_MULTICAN_CONTROLLER_H

#include "os/com/ComStack_Types.h"
#include "hw/dev/can/ConfigTypes.h"
#include "hw/dev/can/HwObjectIdType.h"
#include "hw/dev/can/ControllerIdType.h"

#include "ControllerMapping.h"
#include "ListManager.h"
#include "RegTypes.h"
#include "Addr.h"

namespace hw { namespace dev { namespace can { namespace multican {

class Controller {
public:
	static ControllerIdType
	CtrlUnitToCtrlId(UInt ControllerUnit) {
		switch(ControllerUnit) {
		case 0: return cfHW_DEV_CAN_TC_MULTICAN_CONTROLLER_UNIT_0_MAPPED_ID;
		case 1: return cfHW_DEV_CAN_TC_MULTICAN_CONTROLLER_UNIT_1_MAPPED_ID;
		case 2: return cfHW_DEV_CAN_TC_MULTICAN_CONTROLLER_UNIT_2_MAPPED_ID;
		default:
		case 3: return cfHW_DEV_CAN_TC_MULTICAN_CONTROLLER_UNIT_3_MAPPED_ID;
		}
	}

	static void
	InitAll(ControllerConfigType const * const * ControllerConfigs,
			ControllerIdType ControllerMultiplicity) {
		ControllerIdType i = 0;

		// We assume ControllerMultipliciy <= 4
		for(; i < ControllerMultiplicity; ++i) {
			Inst(i).InitController(ControllerConfigs[i]);
		}

		ControllerMultiplicity_ = i;
	}

	static ControllerIdType
	Multiplicity() {
		return ControllerMultiplicity_;
	}

	static Controller &
	Inst(ControllerIdType ControllerId);

	static Controller &
	InstByUnit(UInt ControllerUnit) {
		return  ((Controller *)
					(HW_DEV_CAN_MULTICAN_MODULE_BASE_ADDRESS
					+ HW_DEV_CAN_MULTICAN_CONTROLLERS_BASE_OFFSET))
				[ControllerUnit];
	}

	static void
	EnableAllCCE() {
		for(UInt i = 0; i < 4; ++i) {
			InstByUnit(i)._NCR.bits.CCE = 1;
		}
	}

	static void
	DisableAllCCE() {
		for(UInt i = 0; i < 4; ++i) {
			InstByUnit(i)._NCR.bits.CCE = 0;
		}
	}

private:
	CAN_NCRm_t 		_NCR;			//!< Node Control Register
	CAN_NSRm_t 		_NSR;			//!< Node Status Register
	CAN_NIPRm_t		_NIPR;			//!< Node Interrupt Pointer Register
	CAN_NPCRm_t		_NPCR;			//!< Node Port Control Register
	CAN_NBTRm_t		_NBTR;			//!< Node Bit Timing Register
	CAN_NECNTm_t	_NECNT;			//!< Node Error Counter Register
	CAN_NFCRm_t		_NFCR;			//!< Node Frame Counter Register

	// Fill the rest with dummy bytes to be able to do correct array indexing into the hardware (64 - 7)
	uint32 dummy[57];

	static ControllerIdType ControllerMultiplicity_;

public:
	inline UInt
	Unit() const {
		return this - ((Controller const *)
				(HW_DEV_CAN_MULTICAN_MODULE_BASE_ADDRESS
				+ HW_DEV_CAN_MULTICAN_CONTROLLERS_BASE_OFFSET));
	}

	inline ControllerIdType
	Id() const {
		return CtrlUnitToCtrlId(Unit());
	}

	inline bool
	GetFirstHwObjHndl(HwObjectIdType * p_HwObjHndl) const {
		return (ListManager::Inst().GetFirstHwObjHndl(Unit() + 1, p_HwObjHndl)
				== ListManager::LIST_MANAGER_OK);
	}

	inline bool
	GetLastHwObjHndl(HwObjectIdType * p_HwObjHndl) const {
		return (ListManager::Inst().GetLastHwObjHndl(Unit() + 1, p_HwObjHndl)
				== ListManager::LIST_MANAGER_OK);
	}

	inline bool
	IsBusOff (void) __attribute__((always_inline)) {
		return (_NSR.bits.BOFF == 1);
	}

	inline void
	AcknowledgeAlert (void) __attribute__((always_inline)) {
		_NSR.reg = ~(1 << CAN_NSRm_ALERT_SHIFT);
	}

	hw::dev::can::ReturnType
	InitController (ControllerConfigType const * p_Config);

	ReturnType
	SetControllerMode (StateTransitionType Transition);

}; // class hw::dev::can::multican::Controller

}}}}

#endif /* HW_DEV_CAN_MULTICAN_CONTROLLER_H */
