// $Id: SysData.h 995 2009-08-11 14:26:31Z mike $
// Created:   24.06.2006 lohmann

/**
 * @file SysData.h
 * @brief Contains the hw::hal::SysData structure 
 */

#ifndef __HW_HAL_SYSDATA__
#define __HW_HAL_SYSDATA__
 
namespace hw { namespace hal {


/**
 * @brief CiAO kernel-internal global data.
 * 
 * SysData is intended as a container for global state that should be
 * low-overhead accessable, such as the currently running thread or 
 * the current IRQL. 
 * SysData is initially empty, the actual data is inserted by aspects.
 * 
 * Internally SysData is put into an own section, which makes it possible
 * to let the linker place it into fast RAM. On some systems, the instance pointer
 * is stored in a global address register.
 * 
 * Aspect should provide accessor functions for the additional data
 * they insert, as some data might only semantically be a part of SysData, 
 * but actuall reside at a hardware-specific address or a special register.
 */
class SysData {
	static SysData theSysData_;
	//! Performs start-time initializations. Only called by advice.
	void init();
  protected:
    SysData(){}

  public:	
	
	
	/* Inst() has to be inserted by an aspect. SysData access should be as efficient as possible,
	 * thus on some plattforms (e.g. TriCore) the instance pointer is held in a global address register.
	 * However, the standard-approach to execution-around-advice an existing Inst() implementation
	 * leads, unfortunately, to some overhead especially on TriCore, as the compiler seems to be not
	 * able to optimize the required tjp->result() access - it always uses the stack. 
	 */

	/* standard-implementation inserted by SysData_Inst aspect. 
	static SysData& Inst() {
		return theSysData_;
	}
	*/
};



}} // namespace hw::hal

 
#endif // __HW_HAL_SYSDATA__
