// $Id: CPU.h 1211 2010-07-16 13:05:25Z sijesche $
// created:   04.03.2006 lohmann
 
/**
 * \file CPU.h
 * \brief TriCore-Version of hw::hal::CPU
 */
 
#ifndef __HW_HAL_CPU_H__
#define __HW_HAL_CPU_H__

#include "hw/triboard1796.h"
#include "stdinc.h"

namespace hw { namespace hal {

typedef int Bool;

/**
 * @brief TC1796 CPU class
 */
CIAO_STD class CPU {
		
  public:


	//! Type to store an IE (interrupts enabled) state.
 	CIAO_STD typedef ICR_t_nonv IEState;
 
	//! Returns status of the IE (interrupt enable) flag.
	CIAO_STD inline static Bool IE(){
		return getICR().bits.IE; 
	}

	//! Disable IRQs globally.
	CIAO_STD inline static void disable() {
		_disable();			
	}

	//! Disable IRQs globally, returns the pervious IE state in /a oldIE.
	CIAO_STD inline static void disable( IEState& oldIE ) {
		oldIE = getICR();
		_disable();			
	}

	//! enable IRQs globally
	CIAO_STD inline static void enable() {
		_enable();
	}
	
	//! Restores the IE state passed in /a oldIE.
	CIAO_STD inline static void enable( IEState oldIE ) {
		if( oldIE.bits.IE ) enable();
	}
 
	//! returns current CPU IRQL
	CIAO_EXT inline static UInt8 getIRQL() {
		return getICR().bits.CCPN;
	}

	//! returns IRQL of a pending IRQ (PIRQ < IRQL) or 0
	CIAO_EXT inline static UInt8 getPIRQL() {
		return getICR().bits.PIPN;
	}
	
	//! returns IRQL of the preceeding context
	CIAO_EXT inline static UInt8 getPreviousIRQL() {
		PCXI_t_nonv myPCXI;
		myPCXI.reg = _mfcr( $pcxi );
		return myPCXI.bits.PCPN;
	}
	
	//! returns the PSW value (program status word)
	inline static PSW_t_nonv getPSW() {
		PSW_t_nonv ret;
		ret.reg = _mfcr( $psw );
		return ret;
	}

	//! returns the ICR value (interrupt control register)
	inline static ICR_t_nonv getICR() {
		ICR_t_nonv ret;
		ret.reg = _mfcr( $icr );
		return ret;
	}
	
	//! modifies the current CPU IRQL
	CIAO_EXT inline static void setIRQL( UInt8 irql) {
		register ICR_t_nonv icr;
		icr.reg = _mfcr( $icr );
		icr.bits.CCPN = irql;
		_mtcr( $icr, icr.reg );
		_isync();
	}
	
};
	
	
} } // namespace hw::hal


#endif // __HW_HAL_CPU_H__
