// $Id: Out0.h 995 2009-08-11 14:26:31Z mike $
// Created:   06.06.2006 lohmann

/**
 * @file Out0.h
 * @brief TC1796 version of hw::hal::Out0
 */

#ifndef __HW_HAL_OUT0_H__
#define __HW_HAL_OUT0_H__

#include "stdinc.h"
#include "Out0Port.h"
 
namespace hw { namespace hal {
 
/**
 * @brief Digital output line 0
 * 
 * The digital output line of Out0 is typically connected to some LED.
 */
class Out0 {
	static Out0 theOut0_;
	//! Performs start-time initializations. Only called by advice.
	void init() {
		port().setMode( PORT_BIT, Port::OUTPUT );
	}
  protected:
    Out0(){}
  public:

	//! Returns the one-and-only Out0 instance
	static Out0& Inst() {
		return theOut0_;
	}
  	
  	//! The internal hw::dev used to implement this line
  	typedef Out0Port Port;
  	
  	//! The used bit number of the internal hw::dev port
  	enum DOXY_SYMBOL(Consts){ PORT_BIT = cfHAL_OUT0_BIT };
  	
  	//! Provides access to the internally used port object
  	const Port& port() const{
  		return Port::Inst();
  	}
  	Port& port() {
  		return Port::Inst();
  	}

	
	//! Sets the output line to high-level
	CIAO_STD void set() {
		port().set( PORT_BIT );
	};
	//! Sets the output line to low-level
	CIAO_STD void clear() {
		port().clear( PORT_BIT );
	};
	//! Reverses the level of the output line
	CIAO_STD void toggle() {
		port().toggle( PORT_BIT );
	};
	//! Returns the level of the output line: low == 0, high != 0
	CIAO_STD Bool get() const {
		return port().get( PORT_BIT );
	};
	//! Returns the level of the output line: low == 0, high != 0
	CIAO_STD operator Bool() const {
		return get();
	}
	//! Sets the output line according to the passed Bool value	
	CIAO_STD Bool operator =( Bool b ) {
		if( b ) set(); else clear();
		return b;
	}
	
};

}}	// namespace hw::hal

 
#endif // __HW_HAL_OUT0_H__
