// $Id: STMTimerBase.h 995 2009-08-11 14:26:31Z mike $
// Created:   22.05.2006 lohmann

/**
 * @file STMTimer.h
 * @brief Timer abstraction for TC1796, based on the STM
 *
 * TC1796 HAL timers are build on top of the TriCore system timer module (STM)
 */

#ifndef __HW_HAL_TIMER_H__
#define __HW_HAL_TIMER_H__

#include "stdinc.h"
#include "hw/dev/STM.h"
#include "hw/triboard1796.h"


namespace hw { namespace hal {

/**
 * @brief Provides some shared definitions for the Timer0 and Timer1 classes
 */
template< class IRQ_TYPE >
class STMTimerBase {
public:

	//! type for the used tricore SRN to trigger interrupts
	typedef IRQ_TYPE IRQ;

	//! type for microseconds values
	typedef UInt32 Usec;
	//! type for values given in internal timer units
	typedef UInt32 Unit;

	//! Timer startup configuration
	enum DOXY_SYMBOL(Consts) {
		FREQ				= hw::dev::STM::FREQ,			//!< internal counter clock frequency
		USEC_FACTOR 		= FREQ / 1000000,				//!< counter units per microsecond
		USEC_ERROR			= FREQ % 1000000,				//!< error (modulo) of above
		USEC_MAX			= 0xffffffff / USEC_FACTOR		//!< maximum wait period in microseconds
	};

	static Unit freq() {
		return hw::dev::STM::freq();
	}
	static Unit usecFactor() {
		return freq() / 1000000;
	}
	static Unit usecError() {
		return freq() % 1000000;
	}
	static Usec usecMax() {
		return 0xffffffff / usecFactor();
	}

	static Unit usec2units( Usec usec ) {
		return usec * usecFactor();
	}
	static Usec units2usec( Unit units ) {
		return units / usecFactor();
	}
};



}} // namespace hw::hal

#endif // __HW_HAL_TIMER_H__
