// $Id: SystemClockImpl.h 995 2009-08-11 14:26:31Z mike $
// Created:   02.06.2006 lohmann

/**
 * @file SystemClockImpl.h
 * @brief Contains the TC1796 implementation base of hw::hal::SystemTimer
 * 
 */


#ifndef __HW_HAL_SystemClockImpl_H__
#define __HW_HAL_SystemClockImpl_H__

#include "stdinc.h"
#include "hw/dev/STM.h"
#include "hw/triboard1796.h"


namespace hw { namespace hal {

/**
 * @brief Implements the hw::hal::SystemTimer on top of the TC1796 hw::dev::STM device.
 * 
 * @tc The SystemClockImpl is based on the 56bit-Timer of hw::dev::STM, which typically
 * runs at full or half CPU clock rate. So timing is pretty precise.
 */
class SystemClockImpl {
  protected:
    // We are just used as base class, so neither allow instantiation nor provide a singleton
    SystemClockImpl(){}
  public:	

	//! Type for a SystemClockImpl time value
	CIAO_STD typedef UInt64 Value;
	
	//! Type for the SystemClockImpl internal frequency
	CIAO_STD typedef UInt32 Frequency;

	//! Returns the number of expired system clock ticks since the system has been started.
	CIAO_STD Value value() const {
		return hw::dev::STM::Inst().value();
	}
	
	//! Returns the frequency of the system clock in Hz.
	CIAO_STD Frequency freq() const {
		return hw::dev::STM::Inst().freq();
	}
	
};



}} // namespace hw::hal

#endif // __HW_HAL_SystemClockImpl_H__
