/*
 * mempool.h
 *
 *  Created on: 06.04.2010
 *      Author: bossel00
 */

#ifndef BASICPAGETABLEMEMPOOL_H_
#define BASICPAGETABLEMEMPOOL_H_

#include "stdinc.h"

namespace hw {
namespace hal {

template<int BLOCKSIZE, int NUMBLOCKS, typename PtrType = void*>
class BasicPageTableMempool {
public:
	static int get_blocksize() {
		return BLOCKSIZE;
	}

private:
	UInt8   pool[BLOCKSIZE * NUMBLOCKS];
	UInt8*  freelist[NUMBLOCKS];
	UInt8** head;

public:
	void init() {
		for (unsigned int i = 0; i < NUMBLOCKS; ++i)
			freelist[i] = pool + i * BLOCKSIZE;

		head = freelist + NUMBLOCKS - 1;
	}

	PtrType alloc(bool clear = true) {
		if (head < freelist)
			return 0;

		UInt8* mem = *head--;

		if (clear) {
			for (int i = 0; i < BLOCKSIZE; ++i) {
				mem[i] = 0;
			}
		}

		return static_cast<PtrType> (static_cast<void*>(mem));
	}

	void free(PtrType mem) {
		*++head = static_cast<UInt8*> (mem);
	}
};

}} // namespace hw::hal

#endif /* BASICPAGETABLEMEMPOOL_H_ */
