#include "OutputDeviceSTDOUT.h"

#include "hw/guest/syscall.h"

#define         ESC_CLEAR             "\033[2J"
#define         ESC_HOME              "\033[1;1H"
#define         ESC_NORMAL            "\033[0m"
#define         ESC_RED               "\033[31m"
#define         ESC_GREEN             "\033[32m"
#define         ESC_YELLOW            "\033[33m"
#define         ESC_BLUE              "\033[34m"
#define         ESC_INTENSITY_BRIGHT  "\033[1m"
#define         ESC_INTENSITY_FAINT   "\033[2m"
#define         ESC_INTENSITY_NORMAL  "\033[22m"

namespace hw {
namespace hal {

OutputDeviceSTDOUT OutputDeviceSTDOUT::theOutputDevice_;

void OutputDeviceSTDOUT::writeStdout(const void* data, unsigned long size) {
	hw::guest::syswrite(1, data, size);
}

void OutputDeviceSTDOUT::sendAttributeEscapeCodes() {
	//                    0 1234567890  1 23456
	char cmdbuffer[] = "\033[3F;4B;IIm\033[?25C";

	char* foreground = cmdbuffer + 3;
	char* background = cmdbuffer + 6;
	char* intensity  = cmdbuffer + 8;
	char* cursor     = cmdbuffer + 16;

	// special case: reset
	if (attribute_.foreground == 0x8 && attribute_.background == 0x8) {
		writeStdout("\033[0m", 4);
		return;
	}

	*foreground = attribute_.foreground + '0';
	*background = attribute_.background + '0';

	if (attribute_.intensity) {
		intensity[0] = '0';
		intensity[1] = '1';
	}
	else {
		intensity[0] = '2';
		intensity[1] = '2';
	}

	if (attribute_.cursor) {
		*cursor = 'h';
	}
	else {
		*cursor = 'l';
	}

	writeStdout(cmdbuffer, 17);
}

void OutputDeviceSTDOUT::clearScreen() {
	writeStdout(ESC_NORMAL ESC_CLEAR ESC_HOME, sizeof(ESC_NORMAL) - 1 + sizeof(ESC_CLEAR) - 1 + sizeof(ESC_HOME) - 1);
}

void OutputDeviceSTDOUT::setCursorPosition(int x, int y) {
	x = x % 1000 + 1;
	y = y % 1000 + 1;

	//                    0 123456789
	char cmdbuffer[] = "\033[yyy;xxxH";

	char* x_str = cmdbuffer + 8;
	char* y_str = cmdbuffer + 4;

	for (int i = 0; i < 3; ++i) {
		*x_str-- = (x % 10) + '0';
		x /= 10;

		*y_str-- = (y % 10) + '0';
		y /= 10;
	}

	writeStdout(cmdbuffer, 10);
}

void OutputDeviceSTDOUT::putCharacter(char c) {
	sendAttributeEscapeCodes();
	writeStdout(&c, 1);
}

void OutputDeviceSTDOUT::putString(const char* str, unsigned long size) {
	sendAttributeEscapeCodes();
	writeStdout(str, size);
}

void OutputDeviceSTDOUT::setAttribute(const OutputDeviceSTDOUT::Attribute& attribute) {
	attribute_ = attribute;
}

void OutputDeviceSTDOUT::resetAttribute() { // reset to default
	attribute_ = Attribute(DEFAULT_ATTRIBUTE);
}

}
}
