/*
 * SystemTimerImpl.h
 *
 *  Created on: Jun 21, 2010
 *      Author: bossel00
 */

#ifndef SYSTEMTIMERIMPL_H_
#define SYSTEMTIMERIMPL_H_

#include "stdinc.h"
#include "hw/dev/PIT.h"
#include "hw/irq/IRQ_PIT.h"


namespace hw { namespace hal {

class SystemTimerImpl {
public:
	typedef hw::irq::IRQ_PIT IRQ;
	typedef UInt64 Ticks;

private:
	static SystemTimerImpl impl_;

	//! Performs start-time initializations. Only called by advice.
	void init();

	Ticks period_;
	bool single_;
	bool running_;

	Ticks timer_;
  
  bool shot_;
  void doAction();

  public:
	//! Returns the one-and-only SystemTimer instance
	static SystemTimerImpl& Inst() {
		return impl_;
	}

	/**
	 * @brief  Windup timer as periodical timer.
	 *
	 * A periodical timer triggers an irq every \a timer units.
	 *
	 */
	CIAO_STD void windupPeriodical( Ticks period ) {
		period_ = period;
		single_ = false;
	}

	/**
	 * @brief  Windup timer as single shot
	 *
	 * A single shot timer triggers an ... TODO
	 */
	CIAO_STD void windupAndStartSingle( Ticks period ) {
		period_ = period + timer_;
		single_ = true;
		running_ = true;
	}


	CIAO_STD void start() {
		running_ = true;
	}

	CIAO_STD void stop() {
		running_ = false;
	}

	/**
	 * @brief Acknowledge timer event. To be called by the IRQ handler (prologue!).
	 *
	 * @todo Maybe we should provide an aspect for this.
	 */
	CIAO_STD void CIAO_INLINE irqAck() {
	}

	bool Handler();

	Ticks getCounter() volatile { return timer_; }

	template<int N> static volatile UInt32 TIM() {
		return (Inst().getCounter() >> N * 4) & 0xFFFFFFFF;
	}
};

} // hal

namespace dev {
typedef hw::hal::SystemTimerImpl STM;
}

} // hw

#endif /* SYSTEMTIMERIMPL_H_ */
