/*
 * PageTable64.h
 *
 *  Created on: 07.04.2010
 *      Author: bossel00
 */

#ifndef PAGETABLE64_H_
#define PAGETABLE64_H_

#include "BasicPageTable.h"

namespace hw {
namespace hal {

namespace paging {

// page table with 4KB pagesize for 32-Bit protected mode with physical address extension
namespace PAE_4KB {
// TODO: Implementieren
} // namespace PAE_4KB


// page table with 2MB pagesize for 32-Bit protected mode with physical address extension
namespace PAE_2MB {
class PDE: public BasicPageTable<void*> {
public:
	PDE(void* mem = 0, bool present = false, bool writeable = false, bool user = false, bool nx = false)
	   : BasicPageTable<void*>(mem, present, writeable, user, 1, nx) {
	}
};

class PDPTE: public BasicPageTable<PDE*> {
public:
	PDPTE(PDE* pde = 0, bool present = false)
	   : BasicPageTable<PDE*>(pde, present, 0, 0, 0, 0) {
	}
};

class PageTable {
	PDPTE pdpte_[4]  __attribute__ ((aligned (4096)));
	PDE pde_[4][512] __attribute__ ((aligned (4096)));

public:
	void init() {

		// zeroing pagetable
		for (int i = 0; i < sizeof(PageTable)/sizeof(unsigned); ++i) {
			((unsigned*) this)[i] = 0;
		}

		for (int pdpte_index = 0; pdpte_index < 4; ++pdpte_index) {

			pdpte_[pdpte_index].setPtr(pde_[pdpte_index]);
			pdpte_[pdpte_index].setPresent(true);

			for (int pde_index = 0; pde_index < 512; ++pde_index) {
				// TODO: restriktivere Rechte setzen
				pde_[pdpte_index][pde_index].setWriteable(true);
				pde_[pdpte_index][pde_index].setUser(true);
				pde_[pdpte_index][pde_index].setPagesize(1);
			}
		}
	}

	PageTable() {
		init();
	}

	PDE* getPDE(void* address) {
		unsigned long addr = (unsigned long) address;
		unsigned long pdpte_index = addr >> 30;
		unsigned long pde_index = (addr >> 21) & 0x1FF;

		return &pde_[pdpte_index][pde_index];
	}
};

} // namespace PAE_2MB


// page table with 4KB pagesize for 64-Bit long mode
namespace LM_4KB {
// TODO: Implementieren
} // namespace LM_4KB


// page table with 2MB pagesize for 64-Bit long mode
namespace LM_2MB {
class PDE: public BasicPageTable<void*> {
public:
	PDE(void* mem, bool present, bool writeable, bool user, bool nx = false)
	   : BasicPageTable<void*>(mem, present, writeable, user, 1, nx) {
	}
};

class PDPE: public BasicPageTable<PDE*> {
public:
	PDPE(PDE* pde, bool present, bool writeable, bool user, bool nx = false)
	   : BasicPageTable<PDE*>(pde, present, writeable, user, 0, nx) {
	}
};

class PML4E : public BasicPageTable<PDPE*> {
public:
	PML4E(PDPE* pdpe, bool present, bool writeable, bool user, bool nx = false)
	   : BasicPageTable<PDPE*>(pdpe, present, writeable, user, 0, nx) {}
};
} // namespace LM_2MB


// page table with 1GB pagesize for 64-Bit long mode
namespace LM_1GB {
// TODO: Implementieren
} // namespace LM_1GB


} // namespace paging

} // namespace hal
} // namespace hw


#endif /* PAGETABLE64_H_ */
