#ifndef __HW_HAL_SystemClockImpl_H__
#define __HW_HAL_SystemClockImpl_H__

#include "stdinc.h"


namespace hw { namespace hal {

/**
 * @brief Implements the hw::hal::SystemTimer on top of the 64 bit time-stamp counter of modern x86 processors.
 *
 */
class SystemClockImpl {
  protected:
    // We are just used as base class, so neither allow instantiation nor provide a singleton
    SystemClockImpl(){}
  public:

	//! Type for a SystemClockImpl time value
	CIAO_STD typedef UInt64 Value;

	//! Type for the SystemClockImpl internal frequency
	CIAO_STD typedef UInt32 Frequency;

	//! Returns the number of expired system clock ticks since the system has been started.
	CIAO_STD Value value() const {
		UInt32 edx, eax;
		asm volatile( "rdtsc" : "=d"(edx), "=a"(eax) );
		return (static_cast<Value>(edx) << 32) | eax;
	}

	//! Returns the frequency of the system clock in Hz.
	CIAO_STD Frequency freq() const {
		// TODO: Taktfrequenz zur Laufzeit bestimmen?
		//       Taktfrequenz kann sich je nach Last zur Laufzeit ändern?
		return 2800000000U;
	}

};



}} // namespace hw::hal

#endif // __HW_HAL_SystemClockImpl_H__
