/*
 * x86_TSSContext.h
 *
 *  Created on: Feb 24, 2010
 *      Author: bossel00
 */

#ifndef X86_TSSCONTEXT_H_
#define X86_TSSCONTEXT_H_

#include "stdinc.h"

#include "x86_TSSContext_flags.h"

namespace hw {
namespace hal {

struct x86Context {
	UInt16 task_link;
	UInt16 reserved0;

	void*  esp0;
	UInt16 ss0;
	UInt16 reserved1;

	void*  esp1;
	UInt16 ss1;
	UInt16 reserved2;

	void*  esp2;
	UInt16 ss2;
	UInt16 reserved3;

	union {
		X86_PAGETABLE cr3_pagetable;                      // object only contains a ptr to a complete pagetable
		X86_PAGETABLE_PTR cr3;
	};

	void*  eip;
	UInt32 eflags;
	UInt32 eax;
	UInt32 ecx;
	UInt32 edx;
	UInt32 ebx;
	void*  esp;
	void*  ebp;
	UInt32 esi;
	UInt32 edi;

	UInt16 es;
	UInt16 reserved4;

	UInt16 cs;
	UInt16 reserved5;

	UInt16 ss;
	UInt16 reserved6;

	UInt16 ds;
	UInt16 reserved7;

	UInt16 fs;
	UInt16 reserved8;

	UInt16 gs;
	UInt16 reserved9;

	UInt16 ldt;
	UInt16 reserved10;

	UInt16 trap;
	UInt16 io_base;

	void clear() {
		task_link  = 0;
		reserved0  = 0;
		esp0       = 0;
		ss0        = 0;
		reserved1  = 0;
		esp1       = 0;
		ss1        = 0;
		reserved2  = 0;
		esp2       = 0;
		ss2        = 0;
		reserved3  = 0;
		cr3        = 0;
		eip        = 0;
		eflags     = 0;
		eax        = 0;
		ecx        = 0;
		edx        = 0;
		ebx        = 0;
		esp        = 0;
		ebp        = 0;
		esi        = 0;
		edi        = 0;
		es         = 0;
		reserved4  = 0;
		cs         = 0;
		reserved5  = 0;
		ss         = 0;
		reserved6  = 0;
		ds         = 0;
		reserved7  = 0;
		fs         = 0;
		reserved8  = 0;
		gs         = 0;
		reserved9  = 0;
		ldt        = 0;
		reserved10 = 0;
		trap       = 0;
		io_base    = 0;
	}

	void fill_segments() {
		asm volatile(
				"movl %%ss, %%eax;"
				"movl %%eax, %0;"
				"movl %%eax, %1;"
				"movl %%eax, %2;"
				"movl %%eax, %6;"
				"movl %%cr3, %%eax;"
				"movl %%eax, %3;"
				"movl %%es, %%eax;"
				"movl %%eax, %4;"
				"movl %%cs, %%eax;"
				"movl %%eax, %5;"
				"movl %%ds, %%eax;"
				"movl %%eax, %7;"
				"movl %%fs, %%eax;"
				"movl %%eax, %8;"
				"movl %%gs, %%eax;"
				"movl %%eax, %9;"
				: "=m"(ss0), "=m"(ss1), "=m"(ss2), "=m"(cr3), "=m"(es), "=m"(cs), "=m"(ss), "=m"(ds), "=m"(fs), "=m"(gs)
				:
				: "%eax", "memory"
				);
	}
}__attribute__((packed));

}
}

#endif /* X86_TSSCONTEXT_H_ */
