/** \author Christian Meier
 *  \version $Id: RxHwObjectType.h 995 2009-08-11 14:26:31Z mike $
 */

#ifndef HW_HAL_CAN_MULTICAN_RX_HW_OBJECT_TYPE_H
#define HW_HAL_CAN_MULTICAN_RX_HW_OBJECT_TYPE_H

#include "os/com/ComStack_Types.h"
#include "hw/hal/can/rx/RxHwObjectType.h"
#include "hw/hal/can/rx/RxHwObjectIdType.h"
#include "hw/hal/can/Cfg.h"

namespace hw { namespace hal { namespace can {

class RxLpduType;

namespace multican {

/**
 *	The class hw::hal::can::multican::RxHwObjectType is the runtime container that is responsible for
 *  the correct delivery mapping to RxLpdus of received CAN frames.
 *  
 *  The class is derived from a base RxHwObjectType that handles the correct filtering.
 *  The data member data is only stored in the base class.
 *  The subclass exists to provide sufficient RxHwObjectType instantiations for each CanDriverImpl
 *  integrated into the Can Interface. For this to work the Init method accuires enough RxLpdu pointer
 *  elements and assigns them to the base member attributes.
 */
class RxHwObjectType : public hw::hal::can::RxHwObjectType {
public:
	inline RxHwObjectIdType
	Id() {
		return this - RxHwObjects_;
	}

private:
	inline void
	Init(RxHwObjectConfigType const * RxHwObjectConfig) {
		if(RxHwObjectConfig->RxLpduIdMultiplicity_ != 0) {
			RxLpdus_ = &EntryPool_[NextEntryIndex_];
			NextEntryIndex_ += RxHwObjectConfig->RxLpduIdMultiplicity_;
			RxLpduMultiplicity_ = RxHwObjectConfig->RxLpduIdMultiplicity_;
			hw::hal::can::RxHwObjectType::Init(RxHwObjectConfig);
		} else {
			RxLpdus_			= NULL;
			RxLpduMultiplicity_ = 0;
		}
	}

	inline RxHwObjectType () {}

public:
	static void
	InitAll(RxHwObjectConfigType const * ConfigArray, RxHwObjectIdType RxHwObjectMultiplicity) {
		RxHwObjectIdType i = 0;
		NextEntryIndex_ = 0;

		// We assume RxHwObjectMultiplicity <= cfHW_DEV_CAN_RX_HW_OBJECT_MULTIPLICITY
		for(; i < RxHwObjectMultiplicity; ++i) {
			RxHwObjects_[i].Init(&ConfigArray[i]);
		}

		RxHwObjectMultiplicity_ = i;
	}

	static RxHwObjectType&
	Inst(RxHwObjectIdType RxHwObjectId) {
		return RxHwObjects_[RxHwObjectId];
	}

	static RxHwObjectIdType
	Multiplicity() {
		return RxHwObjectMultiplicity_;
	}

private:
	// All instances of multican::RxHwObjects
	//
	// Here we simply use cfHW_DEV_MULTICAN_NUM_HW_OBJECTS
	// Although not all are Rx Objects we for now need a
	// object for every Hoh, not only hrh
	static RxHwObjectType RxHwObjects_[cfHW_DEV_CAN_MULTICAN_NUM_HW_OBJECTS];
	static RxHwObjectIdStorageType RxHwObjectMultiplicity_;

	// Pool of RxLpdu pointers that are assigned to each RxHwObject
	static RxLpduType * EntryPool_[cfHW_HAL_CAN_MULTICAN_RX_LPDU_MULTIPLICITY];
	static RxLpduIdStorageType NextEntryIndex_;
};

}

}}}

#endif
