#ifndef HW_IRQ_IRQ_PIT_H_
#define HW_IRQ_IRQ_PIT_H_

#include "stdinc.h"
#include "IRQBase.h"

namespace hw {
namespace irq {

/**
 * @brief Provides access to x86 Programmable Interrupt Timer
 *
 */
class IRQ_PIT: public IRQBase {

	static const UInt8 IRQ_PIT_V = 0;

	typedef IRQBase Super;

	static UInt32 counter_;

public:
	static const UInt8 REGISTER = IRQ_PIT_V;

public:
	static void enable() {
		Super::enableIRQ(IRQ_PIT_V);
	}
	static void disable() {
		Super::disableIRQ(IRQ_PIT_V);
	}

	//! Advice should bind to this
	static Bool Handler();

	//! The actual hardware handler just inlines Handler(), should not be affected by advice.
	static Bool HWHandler();

	static void init() {
		// +32 für Hardware-Unterbrechungen
		handler_table[IRQ_PIT_V + 32] = HWHandler;
		counter_ = 0;
	}

	static UInt32 getCounter() {
		return counter_;
	}
};

} // namespace irq
} // namespace hw

#endif // HW_IRQ_IRQ_PIT_H_
