/** \author Christian Meier
 *  \version $Id$
 *
 *  This file defines the basic AsCom class where every AsCom relevant thing will be added.
 */

#ifndef OS_COM_ASCOM_H
#define OS_COM_ASCOM_H

#include "os/com/ComStack_Types.h"
#include "ConfigTypes.h"

namespace os { namespace com { namespace ascom {

class AsCom {
public:
	/** Gets the one and only instance of AsCom
	 */
	static AsCom&
	Inst ();

	/** Initializes the complete AsCom subsystem with the given Config
	 */
	void
	Init (ConfigType const * Config);

	/** Deinitializes the complete AsCom subsystem
   *
	 * \attention FIXME: Currently not fully implemented
	 */
	void
	Deinit ();

	/** Gets the current state of AsCom
	 */
	Com_StatusType
	GetStatus () const;

	/** Gets the current config id of AsCom
   *
	 * \attention FIXME: Currently not implemented
	 */
	UInt32
	GetConfigurationId () const;

private:
	// Singleton protected constructor
	AsCom () {}

	Com_StatusType State_;
	UInt32 ConfigurationId_;

	static AsCom theAsCom_;
};

inline AsCom&
AsCom::Inst (void) {
	return theAsCom_;
}

inline Com_StatusType
AsCom::GetStatus () const {
	return State_;
}

inline UInt32
AsCom::GetConfigurationId () const {
	return ConfigurationId_;
}

}}}

#endif
