#ifndef UTIL_CONTAINER_INDIRECT_INDEXED_BUFFER_TYPE_H
#define UTIL_CONTAINER_INDIRECT_INDEXED_BUFFER_TYPE_H

#include "os/com/ComStack_Types.h"

namespace os { namespace com {

template <class ELEM_TYPE, int AREA_SIZE, int INST_MULTIPLICITY, typename AREA_SEPARATOR>
class IndirectIndexedBufferType {
public:
	typedef ELEM_TYPE ElemType;
	typedef typename uint_t_value<AREA_SIZE+1>::T ElemIndexStorageType;
	typedef typename uint_t_value_opt<AREA_SIZE+1>::T ElemIndexType;

	static void
	ResetAllocation();

	void
	Allocate(ElemIndexType ElemMultiplicity);

	template <int INST>
	ElemType *
	GetBuffer();

	template <int INST>
	ElemType const *
	GetConstBuffer() const;

private:
	ElemIndexStorageType StartIndex_;

	static ElemType RawBufferArea_[INST_MULTIPLICITY][AREA_SIZE];
	static ElemIndexStorageType NextFree_;
};

template <class ELEM_TYPE, int AREA_SIZE, int INST_MULTIPLICITY, typename AREA_SEPARATOR>
inline void
IndirectIndexedBufferType<ELEM_TYPE, AREA_SIZE, INST_MULTIPLICITY, AREA_SEPARATOR>::Allocate(ElemIndexType ElemMultiplicity) {
	StartIndex_ = NextFree_;
	NextFree_ += ElemMultiplicity;
}

template <class ELEM_TYPE, int AREA_SIZE, int INST_MULTIPLICITY, typename AREA_SEPARATOR>
inline void
IndirectIndexedBufferType<ELEM_TYPE, AREA_SIZE, INST_MULTIPLICITY, AREA_SEPARATOR>::ResetAllocation() {
	NextFree_ = 0;
}

template <class ELEM_TYPE, int AREA_SIZE, int INST_MULTIPLICITY, typename AREA_SEPARATOR>
template <int INST>
inline typename IndirectIndexedBufferType<ELEM_TYPE,AREA_SIZE,INST_MULTIPLICITY,AREA_SEPARATOR>::ElemType *
IndirectIndexedBufferType<ELEM_TYPE, AREA_SIZE, INST_MULTIPLICITY, AREA_SEPARATOR>::GetBuffer() {
	return &RawBufferArea_[INST][StartIndex_];
}

template <class ELEM_TYPE, int AREA_SIZE, int INST_MULTIPLICITY, typename AREA_SEPARATOR>
template <int INST>
inline typename IndirectIndexedBufferType<ELEM_TYPE,AREA_SIZE,INST_MULTIPLICITY,AREA_SEPARATOR>::ElemType const *
IndirectIndexedBufferType<ELEM_TYPE, AREA_SIZE, INST_MULTIPLICITY, AREA_SEPARATOR>::GetConstBuffer() const {
	return &RawBufferArea_[INST][StartIndex_];
}

template <class ELEM_TYPE, int AREA_SIZE, int INST_MULTIPLICITY, typename AREA_SEPARATOR>
typename IndirectIndexedBufferType<ELEM_TYPE,AREA_SIZE,INST_MULTIPLICITY,AREA_SEPARATOR>::ElemType
IndirectIndexedBufferType<ELEM_TYPE,AREA_SIZE,INST_MULTIPLICITY,AREA_SEPARATOR>::RawBufferArea_[INST_MULTIPLICITY][AREA_SIZE];


template <class ELEM_TYPE, int AREA_SIZE, int INST_MULTIPLICITY, typename AREA_SEPARATOR>
typename IndirectIndexedBufferType<ELEM_TYPE,AREA_SIZE,INST_MULTIPLICITY,AREA_SEPARATOR>::ElemIndexStorageType
IndirectIndexedBufferType<ELEM_TYPE,AREA_SIZE,INST_MULTIPLICITY,AREA_SEPARATOR>::NextFree_;

}}

#endif
