#ifndef OS_COM_ASCOM_TX_SIGNAL_TYPE_H
#define OS_COM_ASCOM_TX_SIGNAL_TYPE_H

#include "os/com/ComStack_Types.h"
#include "os/com/ascom/ConfigTypes.h"
#include "util/container/Bitset.h"

namespace os { namespace com { namespace ascom {

class TxIpduType;

class TxSignalType : public TxSignalConfigType {
public:
	static void
	InitAll (TxSignalConfigType const * ConfigArray, TxSignalIdType TxSignalMultiplicity);

	static TxSignalType&
	Inst (TxSignalIdType TxSignalId);

	TxSignalIdType
	Id () const;

	void
	Init (TxSignalConfigType const * ConfigPtr);

	void
	Init ();

	void
	Start (Bool Initialize);

	void
	Stop ();

	StatusType
	SendSignal (const void * SignalDataPtr);

private:
	// Only the static instances inside this class are allowed
	TxSignalType () {}

	StatusType
	ProcessSendSignal (TxIpduType& TxIpdu, void const * SignalDataPtr);

private:
	static TxSignalType TxSignals_[cfOS_COM_ASCOM_TX_SIGNAL_MULTIPLICITY];
};

inline TxSignalIdType
TxSignalType::Id () const {
	return this - TxSignals_;
}

inline void
TxSignalType::Init (TxSignalConfigType const * ConfigPtr) {
	*(TxSignalConfigType*)this = *ConfigPtr;
	Init();
}

inline TxSignalType&
TxSignalType::Inst (TxSignalIdType TxSignalId) {
	return TxSignals_[TxSignalId];
}

inline void
TxSignalType::Start (Bool Initialize) {
	if(Initialize) {
		Init ();
	}
}

inline void
TxSignalType::Stop () {

}

inline void
TxSignalType::InitAll (TxSignalConfigType const * ConfigArray, TxSignalIdType TxSignalMultiplicity) {
	TxSignalIdType i = 0;

	// We assume TxSignalMultiplicity <= cfOS_COM_ASCOM_RX_SIGNAL_MULTIPLICITY
	for (; i < TxSignalMultiplicity; ++i) {
		TxSignals_[i].Init (&ConfigArray[i]);
	}

	for (; i < cfOS_COM_ASCOM_TX_SIGNAL_MULTIPLICITY; ++i) {
		// FIXME Uninit or stop or similar
	}
}

}}}

#endif
