// $Id: SchedStrategy.cpp 1265 2010-09-07 13:47:31Z mike $
// Created:   27.06.2006 lohmann

#include "SchedStrategy.h"
#include "EnterLeave.h"

namespace os { namespace krn {

void SchedStrategy::setRunning( TaskType whom ){
	activeContext_ = running_ 	= whom;
	Task::setState( whom, TASK_RUNNING );
}

void SchedStrategy::setReady( TaskType whom ) {
	ready_.enqueue( whom, Task::getPri( whom ) );
 	Task::setState( whom, TASK_READY );

	// the next few lines effectively implement a full-preemptive system,
	// which is now configurable by an aspect
	if (running_ == INVALID_TASK) { // a task was set READY by an ISR / alarm
		setNeedReschedule ();
	}
}

TaskType SchedStrategy::schedule( Bool prefer_next ) {

	TaskType next;
	
	if( ready_.isEmpty() ) {
		if( prefer_next ) {
			running_ = INVALID_TASK;
			unlock();
			//leaveKernel ();
			idleLoop();
			//enterKernel ();
			lock();
			running_ = activeContext_;
			next = ready_.dequeue();
		}
		else next = getRunning();
	}
	else if( prefer_next ) {
		next = ready_.dequeue();
	}
	else {
		next = getRunning();
		if( ready_.headPri() > Task::getPri( next ) || next == INVALID_TASK ) {
			next = ready_.dequeue();
		}
	}			

	clearNeedReschedule();

	return next;
}

TaskType SchedStrategy::takeNext() {

	TaskType next;
	
	next = getRunning();
	if( !ready_.isEmpty() ) {
		if( ready_.headPri() >= Task::getPri( next ) || next == INVALID_TASK ) {
			next = ready_.dequeue();
		}
	}			

	clearNeedReschedule();

	return next;
}

inline void SchedStrategy::idle() {
	_nop();
}	

void SchedStrategy::idleLoop() {
	while( !needReschedule()) {
		idle();
	}
}

void SchedStrategy::setTaskTable( UInt size ) {
	ASSERT( size <= Task::MAX, size_exceeds_configured_size );
	running_ = INVALID_TASK;
	ready_.init();
	for( UInt i = 0; i < size; ++i ) {
		if( Task::getState( i ) == TASK_READY ) {
			ready_.enqueue( i, Task::getPri( i ) );
		}
	} 	
}

}} // namespace os::krn
