/*
 * Copyright (C) 2008 by egnite GmbH.
 * Copyright (C) 2001-2003 by egnite Software GmbH.
 * Copyright (c) 1993 by Digital Equipment Corporation.
 * Copyright (c) 1983, 1993 by The Regents of the University of California.
 *
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the copyright holders nor the names of
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY EGNITE SOFTWARE GMBH AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL EGNITE
 * SOFTWARE GMBH OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
 * THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * For additional information see http://www.ethernut.de/
 *
 * -
 * Portions Copyright (C) 2000 David J. Hudson <dave@humbug.demon.co.uk>
 *
 * This file is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.
 *
 * You can redistribute this file and/or modify it under the terms of the GNU
 * General Public License (GPL) as published by the Free Software Foundation;
 * either version 2 of the License, or (at your discretion) any later version.
 * See the accompanying file "copying-gpl.txt" for more details.
 *
 * As a special exception to the GPL, permission is granted for additional
 * uses of the text contained in this file.  See the accompanying file
 * "copying-liquorice.txt" for details.
 * -
 * Portions Copyright (c) 1983, 1993 by
 *  The Regents of the University of California.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the University nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * -
 * Portions Copyright (c) 1993 by Digital Equipment Corporation.
 *
 * Permission to use, copy, modify, and distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies, and that
 * the name of Digital Equipment Corporation not be used in advertising or
 * publicity pertaining to distribution of the document or software without
 * specific, written prior permission.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS" AND DIGITAL EQUIPMENT CORP. DISCLAIMS ALL
 * WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS.   IN NO EVENT SHALL DIGITAL EQUIPMENT
 * CORPORATION BE LIABLE FOR ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL
 * DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR
 * PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS
 * ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
 * SOFTWARE.
 */

/*
 * $Log$
 * Revision 1.3  2008/08/11 17:38:27  haraldkipp
 * Added an Ethernet protocol demultiplexer.
 *
 * Revision 1.2  2005/04/08 15:20:50  olereinhardt
 * added <sys/types.h> (__APPLE__) and <netinet/in.h> (__linux__)
 * for htons and simmilar.
 *
 * Revision 1.1.1.1  2003/05/09 14:41:27  haraldkipp
 * Initial using 3.2.1
 *
 * Revision 1.12  2003/02/04 18:14:57  harald
 * Version 3 released
 *
 * Revision 1.11  2002/06/26 17:29:35  harald
 * First pre-release with 2.4 stack
 *
 */

#include "arptable.h"
#include "if_var.h"
#include "stdio.h"
#include "string.h"

#define __byte_swap2(val)           \
    ((((val) & 0xff) << 8) |        \
     (((val) & 0xff00) >> 8))

#define __byte_swap4(val)           \
    ((((val) & 0xff) << 24) |       \
     (((val) & 0xff00) << 8) |      \
     (((val) & 0xff0000) >> 8) |    \
     (((val) & 0xff000000) >> 24))

#define htons(x) __byte_swap2(x)
#define htonl(x) __byte_swap4(x)
#define ntohs(x) __byte_swap2(x)
#define ntohl(x) __byte_swap4(x)

/*!
 * \addtogroup xgArp
 */
/*@{*/

/* 
struct _ARPENTRY {
    uint32_t ae_ip;
    uint8_t ae_ha[6];
    uint8_t ae_flags;
    uint8_t ae_outdated;
};
*/

ARPENTRY os::net::Arptable::table[ARPTABLESIZE];

#define ipAddress(a,b,c,d) ((a<<0)|(b<<8)|(c<<16)|(d<<24))

void os::net::Arptable::Init() {
	uint8_t i;
	for ( i=0; i<ARPTABLESIZE; i++) {
		memset(&table[i], 0, sizeof(ARPENTRY));
		if ( table[i].ae_flags == ATF_REM ) {
			table[i].ae_flags = ATF_EMPTY;
		}
	}
	table[0].ae_flags = ATF_PERM;
	table[0].ae_ip = ipAddress(192,168,34,5);
	table[0].ae_ha[0] = 0x00;
	table[0].ae_ha[1] = 0x40;
	table[0].ae_ha[2] = 0xf4;
	table[0].ae_ha[3] = 0x3c;
	table[0].ae_ha[4] = 0x78;
	table[0].ae_ha[5] = 0x2c;
	table[0].ae_flags = ATF_PERM;
	table[1].ae_ip = ipAddress(192,168,34,11);
	table[1].ae_ha[0] = 0x00;
	table[1].ae_ha[1] = 0x0c;
	table[1].ae_ha[2] = 0xf1;
	table[1].ae_ha[3] = 0x3d;
	table[1].ae_ha[4] = 0x4f;
	table[1].ae_ha[5] = 0x49;
}

void os::net::Arptable::Flush() {
	uint8_t i;
	for ( i=0; i<ARPTABLESIZE; i++) {
		if ( table[i].ae_flags == ATF_REM ) {
			table[i].ae_flags = ATF_EMPTY;
		}
	}
}

void os::net::Arptable::Aging() {
	// FIXME
}

ARPENTRY* os::net::Arptable::Lookup(uint32_t ip) {
	uint8_t i;
	for ( i=0; i<ARPTABLESIZE; i++) {
		if ( os::net::Arptable::table[i].ae_flags != ATF_EMPTY &&
		     os::net::Arptable::table[i].ae_ip == ip ) {
			return &os::net::Arptable::table[i];
		}
	}
	return NULL;
}

ARPENTRY* os::net::Arptable::Add(uint32_t ip, uint8_t* ha) {
	uint8_t i;

	//printf("Adding MAC\n");

	/* Remove outdated entries before adding a new one. */
	os::net::Arptable::Aging();

	for ( i=0; i<ARPTABLESIZE; i++) {
		if ( os::net::Arptable::table[i].ae_flags == ATF_EMPTY ) {

			/* Fill in information */
			os::net::Arptable::table[i].ae_ip = ip;
			memcpy(os::net::Arptable::table[i].ae_ha, ha, 6);
			os::net::Arptable::table[i].ae_flags = ATF_COM;
			os::net::Arptable::table[i].ae_outdated = 0;
			return &os::net::Arptable::table[i];
		}
	}

	return NULL;
}

void os::net::Arptable::Update(uint32_t ip, uint8_t* ha) {
	ARPENTRY* entry = os::net::Arptable::Lookup(ip);

	if ( entry ) {
		if ( ( entry->ae_flags & ATF_PERM ) == 0 ) { 
			entry->ae_outdated = 0;
			memcpy(entry->ae_ha, ha, 6);
			entry->ae_flags |= ATF_COM;
		}
	} else {
		os::net::Arptable::Add(ip, ha);
	}

	for (uint8_t i=0; i<ARPTABLESIZE; i++) {
		if ( table[i].ae_flags != ATF_EMPTY ) {
			//printf("0x%x <--> %x:%x:%x:%x:%x:%x\n", table[i].ae_ip, table[i].ae_ha[0], table[i].ae_ha[1], table[i].ae_ha[2], table[i].ae_ha[3], table[i].ae_ha[4], table[i].ae_ha[5]);
		}
	}
}

int os::net::Arptable::Query(uint32_t ip, uint8_t* mac) {
	ARPENTRY* entry;

	os::net::Arptable::Aging();

	if ( (entry = os::net::Arptable::Lookup(ip) ) ) {
		memcpy(mac,entry->ae_ha,6);
		return 0;
	}

	//FIXME ARP-Discover
	return -1;

}

/*@}*/
