#ifndef __QUEUE_H__
#define __QUEUE_H__

#include "TP_common_queueelement.h"
#include "hw/hal/Timer1.h"
#include "stdio.h"


namespace os { namespace tp {

template <unsigned int N>  
class SortedDeadlineQueue {  
public:
	SortedDeadlineQueue() : first(NULL){};
	void Init();
	inline QueueElement* getFirst();
	QueueElement* insert(UInt32 when);
	void remove(QueueElement* e);
	void printQueue();
private:
	QueueElement array[N];  
	QueueElement* first;
	QueueElement* free;
	enum {CAPACITY = N};
};


template <unsigned int N>
void SortedDeadlineQueue<N>::Init(){
	//initialize free queue
	unsigned int n = 0;
	while (n < (N-1)){
		array[n].next = &array[n+1];
		n++;
	} 
	free = array;
}


template <unsigned int N>
QueueElement* SortedDeadlineQueue<N>::getFirst(){
	return first;
}

template <unsigned int N>
void SortedDeadlineQueue<N>::printQueue(){
	QueueElement* e = first;
	printf("TP: DeadlineHandling: Printing deadline-queue: \r\n");
	while (e != NULL){
		printf("TP: DeadlineHandling: [ %d ] -> ",  hw::hal::Timer1::Inst().units2usec(e->time));
		e = e->next;
	}
	printf("NULL\r\n");
}


template <unsigned int N>
void SortedDeadlineQueue<N>::remove(QueueElement* e){
	if (e == NULL){ // is argument valid?
		return;
	 } else if (e == first){ //element is first
		//remove from queue
		if (e->next != NULL){
			e->next->prev = NULL;
		}
		first = e->next;

		//put element in free queue
		e->next = free;
		free = e;
	} else if (e->next = NULL){ //element is last
		e->prev->next = NULL; //remove element from deadline queue
		 
		//put element in free queue
		e->next = free;
		free = e;
	} else { //element is in the middle of the queue
		e->prev->next = e->next;
		e->next->prev = e->prev;

		//put element in the free queue
		e->next = free;
		free = e;
	}
}


template <unsigned int N>  
QueueElement* SortedDeadlineQueue<N>::insert(UInt32 when) {
	// get first element of the free queue 
	QueueElement* newElement = free;
	if (newElement == NULL){
		return NULL;
	}

	//set new first element of the free queue
	free = free->next;

	//set time in new element
	newElement->time = when;
	
	// insert at correct location
	if (first == NULL){ // if queue is empty, insert first element
		newElement->prev = NULL; 
		newElement->next =NULL; 
		first = newElement;
	} else {  // else, find the correct location for this element
		QueueElement* i = first;
		QueueElement* prev = NULL;
		while ((i != NULL) && (newElement->time > i->time)){ //get correct location
			prev = i;
			i = i->next;
		}

		if (i == first) { //new element has to be placed at the beginning of the queue
			newElement->next = first;
			newElement->prev = NULL;
			first = newElement;
		} else if (i == NULL){ //new element has to be end of queue
			prev->next = newElement;
			newElement->prev = prev;
			newElement->next = NULL;
		} else { //new element has to be placed in the middle of the queue
			prev->next = newElement;
			i->prev = newElement;
			newElement->prev = prev;
			newElement->next = i;
		}
	}
	return newElement;
}


}} //namespaces

#endif 
