// $Id: intrinsics.h 1316 2010-11-26 20:51:37Z mike $
// Created:   26.06.2006 lohmann

/**
 * @file intrinsics.h
 * @brief TriCore intrinsics and assembler macros
 */

#ifndef __INTRINSICS_H____
#define __INTRINSICS_H____
 
// Tricore-gcc specific intrinsics are defined as macros and
// obviously have to end up in the global namespace.
#include <machine/intrinsics.h>


namespace {
	// The special registers A11 (return address) A10 (stack pointer)
	// A8 and A9 (global registers) are never used by the compiler's
	// register scheduling algorithm. Funny enough, we can nevertheless
	// explicitly bind them to and access them as global variables. This
	// is more efficient as the typical asm macros, as the compiler
	// still can choose the most optimal addressing mode.
	//
	// Do not use directly, use _getXX() / _setXX() macros instead!
//	register void* _tc_reg_A8 asm ("a8");
//	register void* _tc_reg_A9 asm ("a9");
//	register void* _tc_reg_A10 asm ("a10");
//	register void* _tc_reg_A11 asm ("a11");
}

		
/* Additional intrinsics for direct register access */
#define _getSP() \
  ({ register void* res; asm volatile ("mov.aa %0,%%sp" : "=a" (res) : : "memory"); res; })
#define _setSP(val) \
  { register void* newval = (void*)(val); asm volatile ("mov.aa %%sp, %0" : : "a" (newval) : "memory"); }

#define _getRA() \
  ({ register void* res; asm ("mov.aa %0,%%a11" : "=a" (res) : : "memory"); res; })
#define _setRA(val) \
  { register void* newval = (void*)(val); asm volatile( "mov.aa %%a11, %0" : : "a" (newval) : "memory"); }

#define _getA8() \
  ({ register void* res; asm volatile ("mov.aa %0,%%a8" : "=a" (res) : : "memory"); res; })
#define _setA8(val) \
  { register void* newval = (void*)(val); asm volatile( "mov.aa %%a8, %0" : : "a" (newval) : "memory"); }

#define _getA9() \
  ({ register void* res; asm volatile ("mov.aa %0,%%a9" : "=a" (res) : : "memory"); res; })
#define _setA9(val) \
  { register void* newval = (void*)(val); asm volatile( "mov.aa %%a9, %0" : : "a" (newval) : "memory"); }

#define _getA1() \
  ({ register void* res; asm volatile ("mov.aa %0,%%a1" : "=a" (res) : : "memory"); res; })
#define _setA1(val) \
  { register void* newval = (void*)(val); asm volatile( "mov.aa %%a1, %0" : : "a" (newval) : "memory"); }

#define _getD15() \
  ({ register long res; asm volatile ("mov %0,%%d15" : "=d" (res) : : "memory"); res; })
#define _setD15(val) \
  { register long newval = (long)(val); asm volatile( "mov %%d15, %0" : : "d" (newval) : "memory"); }

/* push/pop an int32 ("D"ata) or a ptr ("A"ddress) on/from the stack */
#define _pushD(val) \
  { register long newval = (long)(val); asm volatile( "st.w, [+%%sp], -4, %0" : : "d" (newval) : "memory") ; }
#define _popD() \
  ({ register long res; asm volatile ("ld.w %0, [%%sp+], -4" : "=d" (res) : "memory"); res;})
#define _pushA(ptr) \
  { register void* newval = (void*)(ptr); asm volatile( "st.a, [+%%sp], -4, %0" : : "a" (newval) : "memory") ; }
#define _popA() \
  ({ register void* res; asm volatile( "ld.a %0, [%%sp+], -4" : "=a" (res) : : "memory"); res;})

// count leading zeros of a 32 bit value
#define _clz(val)\
  ({ unsigned res; asm("clz %0, %1" : "=d"(res) : "d"(val) ); res;})

// count leading ones of a 32 bit value
#define _clo(val)\
  ({ unsigned res; asm("clo %0, %1" : "=d"(res) : "d"(val) ); res;})
 
#endif // __INTRINSICS_H____
