#ifndef __Array_h__
#define __Array_h__

//#include "mem/Allocator.h"

#include "util/container/StandardIterator.h"
#include "util/types.h"


//! A InitArray
/*! Container with random access
 */
template<class T,int S>
class Array{

// public typedefs and enums
public:
	//! Type definition of instatiated type
	typedef T value_t;
	
	//! Type definition of pointer to intantiated type
	typedef T* pointer_t;
	
	//! Type definition of an index type
	typedef typename index_t_value<S>::T index_t;
	
	//! Type definition of an iterator type
	typedef StandardIterator<T> iterator_t;
	
	//! a short (STL conforming) form of iterator type. 
	typedef iterator_t iterator;
	
	//! size of Array	
	enum{CAPACITY=S};
	
// protected variables	
protected:
		//! Array of content types
		UInt8 _content[ CAPACITY * sizeof(value_t) ];
		
		
	
//public methods
public:
		//! Constructor 
		Array(){}
		
		//! Copy constructor
		Array(Array& array){
			iterator_t source = array.begin();
			iterator_t target = this->begin();
			while (source != array.end()){
				new (static_cast<void*>(&*target)) value_t(*source);
				++target;
				++source;
			}
		}
		
		//! Desturctor
		~Array(){}
		
		//! Get capacity of Array
		/*! 
		 */
		index_t capacity(){
			return CAPACITY;
		}
		
		
		//! Get content of Container
		/*!
		 * \param i is the position of the stored Object
		 */
		value_t& operator[](int i) {
			return (value_t&) _content[i*(sizeof(value_t))];
			
		}
			
		//! Get an iterator which marks the start of the vector
		iterator_t begin(){
			return (pointer_t) _content;
		}
		
		//! Get an iterator which marks the end of the vector
		iterator_t end(){
			return (pointer_t) &(_content[CAPACITY*(sizeof(value_t))]);
		}
};

#endif // __Array_h__
