#ifndef __alloc_List_h__
#define __alloc_List_h__

#include "util/container/Chain.h"
#include "mem/DefaultAllocator.h"

namespace allocating{

//! A List
/*! Container with constant insertion and deletion time.
 *  This container can be instantiated with custom types and/or 
 *  allocators
 */
template<class T,class A = DefaultAllocator,class C = Chain<T> >
class List{
	public:
		
		//! Type definition for instatiated type
		typedef T value_t;
		
		//! Type definition for pointer to intantiated type
		typedef T* pointer_t;
		
		//! Type of the node which is managed by this container
		typedef C node_t;
		
		//! Type of the iterator 
		typedef typename node_t::iterator_t iterator_t;
		
		//! Type of the allocator
		//typedef typename A::alloc_t::template rebind< node_t >::alloc_t alloc_t;
		typedef typename A::alloc_t alloc_t;
	
		//! a short (STL conforming) form of iterator type. 
		typedef iterator_t iterator;
	
	protected:
		//! Pointer to the first list element
		node_t* _front;
		
		//! Allocator Object
		alloc_t _alloc;
	
	public:
	
	
		//! Constructor
		List():_front(0){}
	
		
		//! Copy Constructor 
		/*!
		 * \param list is another list object
		 */
		List(List& list){}

		
		//! Generate a new list element and add it to the beginning of this List
		/*! 
		 * \param value is reference to value of the new element
		 */
		bool push_front(const value_t& value);
		
		//! Generate a new list element and add it to the beginning of this List
		/*! 
		 * \param value is reference to value of the new element
		 */
		void pop_front();
		

		
		//! Get the content first element of this List
		/*!
		 * \return Reference to the value of first the first \ref node_t
		 */
		const value_t& front() const{
			return _front->value();
		}
		
		
		//! Check if \ref List is empty
		/*! 
		 * \return true if \ref List is empty
		 */
		 bool empty()const{
		 	return _front == 0;
		 }
		 
		 void clear(){
		 	while(!empty()){
		 		pop_front();
		 	}
		 }
		
		//! Get an iterator which marks the start of the list
		iterator_t begin(){
			return _front;
		}
		
		//! Get an iterator which marks the last element of the list
		iterator_t last(){
			if(_front != 0){
				return _front->findLast();
			}
		}
		
		//! Get an iterator which marks the end of the list
		iterator_t end(){
				return 0;
		}
		

}; // class List

template<class T,class A,class C>
bool List<T,A,C>::push_front(const value_t& value){
	typename List<T,A,C>::node_t* new_node = static_cast<node_t*>( _alloc.allocate(sizeof(node_t)) );
	if ( new_node == 0 ){
		return false;
	}
	new((void*)new_node) typename List<T,A,C>::node_t(this->_front,value);
	this->_front=new_node;
	return true;
}

template<class T,class A,class C>
void List<T,A,C>::pop_front(){
	if(this->_front == 0){
		return;
	}
	typename List<T,A,C>::node_t* tmp = this->_front;
	this->_front=(this->_front)->getNext();
	_alloc.free(static_cast<typename alloc_t::pointer_t >(tmp));
}



}// namespace allocating

#endif // __alloc_List_h__

