#ifndef __Vector_h__
#define __Vector_h__

//#include "mem/Allocator.h"

#include "util/container/StandardIterator.h"
#include "mem/DefaultAllocator.h"
#include "util/types.h"


//! A Vector
/*! Container with efficient random access
 */
template<class T,class A = typename DefaultAllocator::alloc_t >
class Vector{

// public typedefs and enums
public:
	//! Type definition of instatiated type
	typedef T value_t;
	//! Type definition of pointer to intantiated type
	typedef T* pointer_t;

	//! Type definition of an iterator type
	typedef StandardIterator<T> iterator_t;
	
	//! Type of the allocator
	typedef typename A::alloc_t alloc_t;
	
	//! a short (STL conforming) form of iterator type. 
	typedef iterator_t iterator;
	
// protected variables	
protected:
		//! Allocator
		alloc_t _alloc;

		//! Array of value types
		pointer_t _value;
		
		//! size
		int _size;
		
		
	
//public methods
public:
		//! Constructor
		Vector(int size):_size(size){
			_value=_alloc.allocate(size);
		}
		
		//! Constructor
		Vector(alloc_t& allocator):_alloc(allocator),_size(0){};
		
		//! Constructor
		Vector(int size,alloc_t& allocator):_alloc(allocator),_size(size){
			_value=_alloc.allocate(size);
		}
			
		
		
		//! Get value of Container
		/*!
		 * \param i is the position of the stored Object
		 */
		T& operator[](int i) {
			return _value[i];
		}
			
		//! Get an iterator which marks the start of the vector
		iterator_t begin(){
			return _value;
		}
		
		//! Get an iterator which marks the end of the vector
		iterator_t end(){
			return &(_value[_size]);
		}
};

#endif // __Vector_h__
