#ifndef __METAPROG_H__
#define __METAPROG_H__

// This namespace contains several things needed
// for template meta programming
namespace metaprog {
	// Structure that represents "false"
	struct false_ {
		enum { RES = false };
	};

	//	Structure that represents "true"
	struct true_ {
		enum { RES = true };
	};

	// Template that checks whether to types are the same or not.
	template <typename T1, typename T2>
	struct same_types {
		enum { RES = false };
	};

	template <typename T1>
	struct same_types<T1,T1> {
		enum { RES = true };
	};

	// If the first template parameter resolves to true
	// the first type is returned in T, else the second type
	template< bool FIRST, class A, class B >
	struct select
	{
		typedef A T;
	};

	template< class A, class B >
	struct select< false, A, B >
	{
		typedef B T;
	};

	// if_ represents a generic meta prog conditional statement
	template<bool COND_, typename TYPE_, TYPE_ THEN_, TYPE_ ELSE_>
	struct if_ {
		static const TYPE_ RES = THEN_;
	};

	template<typename TYPE_, TYPE_ THEN_, TYPE_ ELSE_>
	struct if_<false,TYPE_,THEN_,ELSE_> {
		static const TYPE_ RES = ELSE_;
	};

	// Evaluates to true if A < B
	template< int A, int B >
	struct less {
		static const bool RES = A < B;
	};

	// Evaluates to true if A > B
	template< int A, int B >
	struct greater {
		static const bool RES = A > B;
	};

	// Calculates the minimum of A and B
	template< int A, int B > struct min {
		enum { RES = A < B ? A : B };
	};

	// Calculates the maximum of A and B
	template< int A, int B > struct max {
		enum { RES = A > B ? A : B };
	};

	// Calculates (2 to the power of the input) - 1s
	template<int I>
	struct TwoPowI_minus1 {
		enum { RES = TwoPowI_minus1< I-1 >::RES + (1<<(I-1)) };
	};

	template<>
	struct TwoPowI_minus1<0> {
		enum { RES = 0 };
	};

	// Calculates the maximum of 0 and the bottom
	// rounded logarithm of the input to the base 2
	template<unsigned int VAL>
	struct log2_bottom {
		enum { RES = log2_bottom< VAL/2 >::RES + 1 };
	};

	template<>
	struct log2_bottom<1> {
		enum { RES = 0 };
	};

	template<>
	struct log2_bottom<0> {
		enum { RES = 0 };
	};

	// Calculates the maximum of 0 and the top
	// rounded logarithm of the input to the base 2
	template<unsigned int VAL>
	struct log2_top  {
		enum { RES = log2_bottom<VAL-1>::RES + 1 };
	};

	template<>
	struct log2_top<1> {
		enum { RES = 0 };
	};

	template<>
	struct log2_top<0> {
		// FIXME normally undefined
		enum { RES = 0 };
	};

	// Calculates how many bits are needed to represent
	// a specific unsigned value VAL
	template<unsigned int VAL>
	struct bitlength {
		enum { RES = log2_bottom< VAL >::RES + 1 };
	};
}	// namespace metaprog

#endif // __METAPROG_H__

