/** \author Christian Meier
 *  \version $Id$
 *
 *  This test checks whether the automatic length check of CAN-LPDUs
 *  is working. The testcase should send one CAN-LPDU with the correct
 *  length, and the second should not be received, but an DlcError
 *  should be raised.
 */

#include "appcode.h"
#include "as/AS.h"
#include "objects.h"
#include "stdio.h"

#include "hw/dev/STM.h"
#include "canif_Cfg.h"

volatile bool GotDlcError = false;
volatile bool ReceivedSomething = false;

UInt8 txBuf[9] = {"12345678"};

os::com::PduInfoType TxPduInfo = {txBuf, 8};

void Alpha::functionTaskTask0(void) {
	AS::CanIf_SetControllerMode(0, hw::hal::can::CANIF_CS_STARTED);
	AS::CanIf_SetControllerMode(1, hw::hal::can::CANIF_CS_STARTED);

	AS::CanIf_Transmit(CANIF_TX_LPDU_TESTDATA0, &TxPduInfo);

	while(!ReceivedSomething);

	TxPduInfo.SduLength_ = 7;
	AS::CanIf_Transmit(CANIF_TX_LPDU_TESTDATA0, &TxPduInfo);

	while(!GotDlcError);

	printf("Everything is OK, a Dlc Error was received!\r\n");

	AS::CanIf_SetControllerMode(0, hw::hal::can::CANIF_CS_STOPPED);
	AS::CanIf_SetControllerMode(1, hw::hal::can::CANIF_CS_STOPPED);

	AS::TerminateTask();
}
