/** \author Christian Meier
 *  \version $Id$
 *
 *  This test checks whether the rx buffer is accessed atomically.
 *  The buffer should always be filled with 8 times the same character.
 */

#include "appcode.h"
#include "as/AS.h"
#include "objects.h"
#include "stdio.h"

#include "canif_Cfg.h"

UInt8 txBuf[10][9] = {"00000000", "11111111", "22222222", "33333333", "44444444",
					 "55555555", "66666666", "77777777", "88888888", "99999999"};

UInt8 rxBuf[9] = "00000000";

os::com::PduInfoType TxPduInfo = {NULL, 8};
os::com::PduInfoType RxPduInfo = {rxBuf, 8};

void SmallDelay() {
	UInt32 TIMValFirst = hw::dev::STM::TIM<4>();
	while ((hw::dev::STM::TIM<4>() - TIMValFirst) > 10);
}

static UInt8 txBufIndex = 9;

void Send() {
	++txBufIndex;
	if(txBufIndex > 9) {
		txBufIndex = 0;
	}
	TxPduInfo.SduDataPtr_ = txBuf[txBufIndex];

	AS::CanIf_Transmit(CANIF_TX_LPDU_TESTDATA0, &TxPduInfo);
}

void Alpha::functionTaskTask0(void) {
	AS::CanIf_SetControllerMode(0, hw::hal::can::CANIF_CS_STARTED);
	AS::CanIf_SetControllerMode(1, hw::hal::can::CANIF_CS_STARTED);

	while(true) {
		Send();
		AS::CanIf_ReadRxPduData(CANIF_RX_LPDU_TESTDATA0, &RxPduInfo);
		Send();

		if(rxBuf[0] != rxBuf[7]) {
			printf("Error occured!\r\n");
			break;
		}

		Send();
		printf("%s\r\n", RxPduInfo.SduDataPtr_);
	}

	// Should never come here

	AS::CanIf_SetControllerMode(0, hw::hal::can::CANIF_CS_STOPPED);
	AS::CanIf_SetControllerMode(1, hw::hal::can::CANIF_CS_STOPPED);

	AS::TerminateTask();
}
