#include "appcode.h"
#include "string.h"
#include "stdio.h"
#include "stdint.h"
#include "as/AS.h"
#include "hw/dev/LAN91.h"
#include "os/net/ethernet.h"
#include "os/net/message.h"
#include "os/net/arptable.h"
#include "os/krn/Messageport.h"
#include "messageport_objects.h"
#include "objects.h"

/* Mac adress definition for enc28j60 */
#define ETHADDR0     0x00
#define ETHADDR1     0x02
#define ETHADDR2     0x3f
#define ETHADDR3     0x17
#define ETHADDR4     0xd5
#define ETHADDR5     0x60
/* Mac adress definition for uip */
#define UIP_ETHADDR0    ETHADDR0
#define UIP_ETHADDR1    ETHADDR1
#define UIP_ETHADDR2    ETHADDR2
#define UIP_ETHADDR3    ETHADDR3
#define UIP_ETHADDR4    ETHADDR4
#define UIP_ETHADDR5    ETHADDR5

static os::net::Message msg;
#define __byte_swap2(val)           \
    ((((val) & 0xff) << 8) |        \
     (((val) & 0xff00) >> 8))

#define __byte_swap4(val)           \
    ((((val) & 0xff) << 24) |       \
     (((val) & 0xff00) << 8) |      \
     (((val) & 0xff0000) >> 8) |    \
     (((val) & 0xff000000) >> 24))

#define htons(x) __byte_swap2(x)
#define htonl(x) __byte_swap4(x)
#define ntohs(x) __byte_swap2(x)
#define ntohl(x) __byte_swap4(x)

#include "hw/irq/DMA_SYSSRC2.h"

void A::functionTaskT1(void) {
	uint16_t len; 
	printf("Task A::T1: started\r\n");
	hw::dev::LAN91::NicInit();
	printf("Task A::T1: NicCompleted\r\n");


	/* configure P1.0 for use as external interrupt pin REQ0 */
	EICR0.bits.EXIS0 = 0;	/* input line IN00 */
	EICR0.bits.REN0  = 1;	/* rising edge */
	EICR0.bits.LDEN0 = 1;	/* automatic reset by falling edge */
	EICR0.bits.EIEN0 = 1;	/* external interrupt enable */
	EICR0.bits.INP0  = 0;	/* interrupt node pointer INT00 */

	hw::irq::DMA_SYSSRC2::enable();

	os::net::Arptable::Init();

	os::net::Message* toSend;

	while ( 1 ) {
		len = hw::dev::LAN91::NicGetPacket(
			static_cast<uint8_t *>(msg.data),
			static_cast<uint16_t>(sizeof(msg.data)));
		printf("A::length read: %d\r\n", len);
		if ( len > 0 ) {
			msg.size = len;
			printf("Start packet (%d) processing\r\n", len);
			os::net::Ethernet::Input(msg);
			printf("End packet processing\r\n");
		}

		toSend = MyMsgPortRe_receiver.receiveMessage();
		printf("A::receiveMessage returned: %x\r\n", toSend);
		if ( toSend != 0x0 ) {
			os::net::Udp::Output(toSend->ip.ip_src, htons(toSend->udp.header.uh_sport), *toSend);
			MyMsgPortRe_receiver.releaseMessage(toSend);
		}

		printf("A::going to sleep\r\n");
		AS::WaitEvent(NetworkHardwareEvent | PacketWaitingEvent);
		printf("A::wakeup call\r\n");
		AS::ClearEvent(PacketWaitingEvent | NetworkHardwareEvent);
	}

	AS::TerminateTask();
}

void A::functionISRNetworkISR() {
//	printf("\r\nInterrupt\r\n");
	AS::SetEvent(T1,NetworkHardwareEvent);
}

void B::functionTaskT2(void) {
	while ( 1 ) {
		printf("B::going to sleep\r\n");
		AS::WaitEvent(PacketWaitingEvent);
		printf("B::wakeup call\r\n");
		AS::ClearEvent(PacketWaitingEvent);
		os::net::Message* msgP = MyMsgPort_receiver.receiveMessage();
		printf("B::getMsg() returned: %x\r\n", (void*)msgP);
		if ( msgP != 0x0 ) {
			printf("Got from the net: >>%s<<\r\n", msgP->udp.data);
			// Echoing

			os::net::Message* response = MyMsgPortRe_sender.allocateMessage();
			if ( response != 0x0 ) {
				memcpy(response, msgP, sizeof(os::net::Message));
				MyMsgPortRe_sender.sendMessage(response);
				AS::SetEvent(T1, PacketWaitingEvent);
			}

			MyMsgPort_receiver.releaseMessage(msgP);
		}
	}
	AS::TerminateTask();
}

