/*
 * Copyright (C) 2008 by egnite GmbH.
 * Copyright (C) 2001-2003 by egnite Software GmbH.
 * Copyright (c) 1993 by Digital Equipment Corporation.
 * Copyright (c) 1983, 1993 by The Regents of the University of California.
 *
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the copyright holders nor the names of
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY EGNITE SOFTWARE GMBH AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL EGNITE
 * SOFTWARE GMBH OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
 * THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * For additional information see http://www.ethernut.de/
 *
 * -
 * Portions Copyright (C) 2000 David J. Hudson <dave@humbug.demon.co.uk>
 *
 * This file is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.
 *
 * You can redistribute this file and/or modify it under the terms of the GNU
 * General Public License (GPL) as published by the Free Software Foundation;
 * either version 2 of the License, or (at your discretion) any later version.
 * See the accompanying file "copying-gpl.txt" for more details.
 *
 * As a special exception to the GPL, permission is granted for additional
 * uses of the text contained in this file.  See the accompanying file
 * "copying-liquorice.txt" for details.
 * -
 * Portions Copyright (c) 1983, 1993 by
 *  The Regents of the University of California.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the University nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * -
 * Portions Copyright (c) 1993 by Digital Equipment Corporation.
 *
 * Permission to use, copy, modify, and distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies, and that
 * the name of Digital Equipment Corporation not be used in advertising or
 * publicity pertaining to distribution of the document or software without
 * specific, written prior permission.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS" AND DIGITAL EQUIPMENT CORP. DISCLAIMS ALL
 * WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS.   IN NO EVENT SHALL DIGITAL EQUIPMENT
 * CORPORATION BE LIABLE FOR ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL
 * DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR
 * PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS
 * ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
 * SOFTWARE.
 */

/*
 * $Log$
 * Revision 1.3  2008/08/11 17:38:27  haraldkipp
 * Added an Ethernet protocol demultiplexer.
 *
 * Revision 1.2  2005/04/08 15:20:50  olereinhardt
 * added <sys/types.h> (__APPLE__) and <netinet/in.h> (__linux__)
 * for htons and simmilar.
 *
 * Revision 1.1.1.1  2003/05/09 14:41:27  haraldkipp
 * Initial using 3.2.1
 *
 * Revision 1.12  2003/02/04 18:14:57  harald
 * Version 3 released
 *
 * Revision 1.11  2002/06/26 17:29:35  harald
 * First pre-release with 2.4 stack
 *
 */

#include "ip.h"
#include "in.h"
#include "icmp.h"
#include "message.h"
#include "hw/dev/LAN91.h"
#include "stdio.h"
#include "string.h"
#include "ethernet.h"
#include "arp.h"
#include "arptable.h"
#include "udp.h"

#define __byte_swap2(val)           \
    ((((val) & 0xff) << 8) |        \
     (((val) & 0xff00) >> 8))

#define __byte_swap4(val)           \
    ((((val) & 0xff) << 24) |       \
     (((val) & 0xff00) << 8) |      \
     (((val) & 0xff0000) >> 8) |    \
     (((val) & 0xff000000) >> 24))

#define htons(x) __byte_swap2(x)
#define htonl(x) __byte_swap4(x)
#define ntohs(x) __byte_swap2(x)
#define ntohl(x) __byte_swap4(x)

/*!
 * \addtogroup xgEthernet
 */
/*@{*/

uint16_t os::net::Ip::pkt_id=0;

void os::net::Ip::Input(Message &msg)
{
	uint_fast8_t bcast;

	printf("os::net::Ip::Input(Message &msg)\r\n");

	/*
	 * Silently discard datagrams of different IP version as well as
	 * fragmented or filtered datagrams.
	 */
	if (msg.ip.ip_v != IPVERSION ||        /* Version check. */
		(ntohs(msg.ip.ip_off) & (IP_MF | IP_OFFMASK)) != 0 ) {      /* Fragmentation. */
		return;
	}

	/*
	 ** IP header length is given in 32-bit fields. Calculate the size in
	 ** bytes and make sure that the header we know will fit in. Check
	 ** further, that the header length is not larger than the bytes we
	 ** received.
	 */

	uint8_t headerLength = msg.ip.ip_hl*4;
	if ( headerLength < sizeof(IPHDR) || headerLength > msg.size ) {
		printf("IP: header too small\r\n");
		return;
	}

	/*
	 * Check for broadcast.
	 */
	if ( msg.ip.ip_dst == INADDR_BROADCAST ) {
		bcast = 1;
	}
	/*
	 * Packet is unicast.
	 */
	else {
		bcast = 0;
		if ((msg.ip.ip_dst == 0 || msg.ip.ip_dst != htonl(hw::dev::NIC::getIP()))) {
			printf("IP: Not for me\r\n");
			return;
		}

		// Put ha in arptable
		os::net::Arptable::Update(msg.ip.ip_src, msg.eth.ether_shost);
	}

	/* Check the IP data length. */
	if ( msg.size < htons(msg.ip.ip_len) ) {
		printf("IP: message too small vor ip data length\r\n");
		return;
	}

	msg.size-=sizeof(IPHDR);

	switch (msg.ip.ip_p) {
		case IPPROTO_ICMP:
			os::net::Icmp::Input(msg);
			break;
		case IPPROTO_UDP:
			os::net::Udp::Input(msg);
			break;
		default:
			/* Unkown protocol, send ICMP destination (protocol)
			* unreachable message.
			*/
			if ( ! bcast ) {
				printf("Sending ICMP Message\r\n");
				os::net::Icmp::Response(ICMP_UNREACH, ICMP_UNREACH_PROTOCOL, 0, msg);
			}
	}
}


uint16_t os::net::Ip::PartialChecksum(uint16_t ics, const void* buf, int len) {
	uint32_t chk = ics;
	const char* data = (const char*)buf;

	while ( len > 1 ) {
		chk += (*((uint16_t*) data));
		len-=2;
		data+=2;
	}

	if ( len == 1 ) {
		chk += (*((uint8_t*) data));
	}

	chk = (chk>>16) + (chk & 0xffff);
	chk += (chk>>16);

	return chk;
}

uint16_t os::net::Ip::Checksum(uint16_t ics, const void* buf, int len) {
	return ~PartialChecksum(ics, buf, len);
}

/*!
 * \brief Send IP datagram.
 *
 * Route an IP datagram to the proper interface.
 *
 * The function will not return until the data has been stored 
 * in the network device hardware for transmission. If the 
 * device is not ready for transmitting a new packet, the 
 * calling thread will be suspended until the device becomes 
 * ready again. If the hardware address of the target host needs 
 * to be resolved the function will be suspended too.
 *
 * \param proto Protocol type.
 * \param dest  Destination IP address. The function will determine
 *              the proper network interface by checking the routing
 *              table. It will also perform any neccessary hardware
 *              address resolution.
 * \param nb    Network buffer structure containing the datagram. 
 *              This buffer will be released if the function returns
 *              an error.
 *
 * \return 0 on success, -1 otherwise.
 *
 * \bug Broadcasts to multiple network devices will fail after the
 *      first device returns an error.
 */
int os::net::Ip::Output(uint8_t proto, uint32_t dest, Message& msg)
{
	uint8_t ha[6];

	/*
	 * Set those items in the IP header, which are common for
	 * all interfaces.
	 */
	msg.ip.ip_v = 4;
	msg.ip.ip_hl = sizeof(IPHDR) / 4;
	msg.ip.ip_tos = 0;
	msg.ip.ip_len = htons(sizeof(IPHDR) + msg.size);
	msg.ip.ip_off = 0;
	msg.ip.ip_ttl = 0x40;
	msg.ip.ip_p = proto;
	msg.ip.ip_dst = dest;

	msg.ip.ip_id = htons(pkt_id++);
	msg.ip.ip_src = htonl(hw::dev::NIC::getIP());
	msg.ip.ip_sum=0;
	msg.ip.ip_sum = Checksum(0, &msg.ip, sizeof(IPHDR));

	msg.size += sizeof(IPHDR);

	if (dest == 0xffffffff) {
		memset(ha, 0xff, sizeof(ha));
		return os::net::Ethernet::Output(ETHERTYPE_IP, ha, msg);
	}

	/*
	 * On Ethernet we query the MAC address of our next hop,
	 * which might be the destination or the gateway to this
	 * destination.
	 */
	// FIXME: Networkmask
	if ( os::net::Arptable::Query(dest, ha) ) {
		/* Note, that a failed ARP request is not considered a 
		 * transmission error. It might be caused by a simple 
		 * packet loss. */
		printf("No entry in arptable found for 0x%x...discarding\n", dest);
		return -1;
	}

	return os::net::Ethernet::Output(ETHERTYPE_IP, ha, msg);
}

/*@}*/
