#include <tut.h>
#include "util/container/allocating/DlList.h"
using namespace allocating;
#include "mem/StaticAllocator.h"
#include <iostream>


namespace tut
{
	
  	
  // Data used by each test
  struct alloc_dllist_basic
  { 
  	enum{SIZE=3};
  	typedef int value_t;
  	typedef StaticAllocator< DlList<value_t>::node_t,SIZE,0> CustomAllocator;
	typedef DlList<value_t,CustomAllocator> IntList;
    IntList list;
    ~alloc_dllist_basic(){list.clear();}
  };

  // Test group registration
  typedef test_group<alloc_dllist_basic> factory;
  typedef factory::object object;
}

namespace
{
  tut::factory tf("allocating dllist basic operations");
}

namespace tut
{
	
  /**
   * Checks empty operation
   */
  template<>
  template<>
  void object::test<1>()
  {
    ensure("empty list",list.empty());
  }
  
  /**
   * Checks push_front and pop_front operation
   */
  template<>
  template<>
  void object::test<2>()
  {
    ensure("push_front succeeds",list.push_front(100));
    ensure("list not empty",!list.empty());
    ensure("list.front.value==100",list.front()==100);
    list.pop_front();
    ensure("list empty",list.empty());
  }
  
  /**
   * Checks pop_front operation
   */
  template<>
  template<>
  void object::test<3>()
  {
	ensure("push_front succeeds",list.push_front(100));
	ensure("list.front.value==100",list.front()==100);
	list.pop_front();
    ensure("list empty",list.empty());
  }
  
  /**
   * Check iterator increment operation
   */
  template<>
  template<>
  void object::test<4>()
  {
  	ensure("push_front succeeds 1",list.push_front(100));
  	ensure("push_front succeeds 2",list.push_front(200));
  	
	IntList::iterator item(list.begin());
	ensure("iterator value",*item == 200);
	++item;
	ensure("incremented iterator value",*item == 100);
	ensure("last iterator value",*(list.last()) == 100);
	
  }
  
  /**
   * Check iterator 
   */
  template<>
  template<>
  void object::test<5>()
  {
  	
  	for(int i=0;i<SIZE;++i){
  		ensure("push_front succeeds",list.push_front(i));
  	}
  	
	IntList::iterator item(list.begin());
	
	int i=SIZE;
	for(item;item != list.end();++item){
		ensure("iterate values",*item == --i);
	}
	
  }
  
  /**
   * Check iterator decrement operation
   */
  template<>
  template<>
  void object::test<6>()
  {
  	for(int i=0;i<SIZE;++i){
  		ensure("push_front succeeds",list.push_front(i));
  	}
  	
	IntList::iterator item(list.last());
	
	int i=0;
	for(item;item != list.end();--item){
 		ensure("iterate values",*item == i++);
	}
	
  }
  
  
}
