#include <tut.h>

#include "util/container/allocating/List.h"


namespace tut
{
	
  // Data used by each test
  struct alloc_list_basic
  { 
  	enum{SIZE=3};
  	typedef int value_t;	   
	typedef allocating::List<value_t> IntList;
    IntList list;
    //always clear list
    ~alloc_list_basic(){list.clear();}
  };

  // Test group registration
  typedef test_group<alloc_list_basic> factory;
  typedef factory::object object;
}

namespace
{
  tut::factory tf("allocating list basic operations");
}

namespace tut
{
	
   /**
   * Checks empty operation
   */
  template<>
  template<>
  void object::test<1>()
  {
    ensure("empty list",list.empty());
  }
  
  /**
   * Checks push_front and pop_front operation
   */
  template<>
  template<>
  void object::test<2>()
  {
    ensure("push_front succeeds",list.push_front(100));
    ensure("list not empty",!list.empty());
    ensure("list.front.value==100",list.front()==100);
    list.pop_front();
    ensure("list empty",list.empty());
  }
  
  /**
   * Checks pop_front operation
   */
  template<>
  template<>
  void object::test<3>()
  {
	ensure("push_front succeeds",list.push_front(100));
	ensure("list.front.value==100",list.front()==100);
	list.pop_front();
    ensure("list empty",list.empty());
  }
  
  /**
   * Check iterator increment operation
   */
  template<>
  template<>
  void object::test<4>()
  {
  	ensure("push_front succeeds 1",list.push_front(100));
  	ensure("push_front succeeds 2",list.push_front(200));
  	
	IntList::iterator item(list.begin());
	ensure("iterator value",*item == 200);
	++item;
	ensure("incremented iterator value",*item == 100);
	ensure("last iterator value",*(list.last()) == 100);
  }
  
  /**
   * Check iterator dereference operation
   */
  template<>
  template<>
  void object::test<5>()
  {
  	
  	for(int i=0;i<SIZE;++i){
  		ensure("push_front succeeds",list.push_front(i));
  	}
  	
	IntList::iterator item(list.begin());
	
	int i=SIZE;
	for(item;item != list.end();++item){
		ensure("iterate values",*item == --i);
	}
  }
}
