#include <tut.h>
#include "util/container/allocating/Queue.h"
using namespace allocating;
#include "mem/StaticAllocator.h"
#include <iostream>


namespace tut
{
	
  	
  // Data used by each test
  struct alloc_queue_basic
  { 
  	enum{SIZE=3};
  	typedef int value_t;
  	typedef StaticAllocator<Queue<value_t>::node_t,SIZE> CustomAllocator;
	typedef Queue<value_t,CustomAllocator> IntQueue;
    IntQueue queue;
    ~alloc_queue_basic(){queue.clear();}
  };

  // Test group registration
  typedef test_group<alloc_queue_basic> factory;
  typedef factory::object object;
}

namespace
{
  tut::factory tf("allocating queue basic operations");
}

namespace tut
{
	
   /**
   * Checks empty operation
   */
  template<>
  template<>
  void object::test<1>()
  {
    ensure("empty queue",queue.empty());
  }
  
  /**
   * Checks push_front operration
   */
  template<>
  template<>
  void object::test<2>()
  {
    ensure("push_front succeeds",queue.push_front(100));
    ensure("queue not empty",!queue.empty());
    ensure("queue.front.value==100",queue.front()==100);
    ensure("queue.back.value==100",queue.back()==100);
  }
  
  /**
   * Checks pop_front operation
   */
  template<>
  template<>
  void object::test<3>()
  {
	ensure("push_front succeeds",queue.push_front(100));
	queue.pop_front();
    ensure("queue empty",queue.empty());
  }
  
   /**
   * Checks push_back operration
   */
  template<>
  template<>
  void object::test<4>()
  {
    ensure("push_back succeeds",queue.push_back(100));
    ensure("queue not empty",!queue.empty());
    ensure("queue.back.value==100",queue.back()==100);
    ensure("queue.back.value==100",queue.front()==100);
  }
  
  /**
   * Checks pop_back operation
   */
  template<>
  template<>
  void object::test<5>()
  {
	ensure("push_back succeeds",queue.push_back(100));
	queue.pop_back();
    ensure("queue empty",queue.empty());
  }
  
  
  /**
   * Check begin/last iterator 
   */
  template<>
  template<>
  void object::test<6>()
  {
  	ensure("push_front succeeds 1",queue.push_front(100));
  	ensure("push_front succeeds 2",queue.push_front(200));
  	
	IntQueue::iterator item(queue.begin());
	ensure("iterator value",*item == 200);
	++item;
	ensure("incremented iterator value",*item == 100);
	ensure("last iterator value",*(queue.last()) == 100);
	
  }
  
  /**
   * Check iterator  operation
   */
  template<>
  template<>
  void object::test<7>()
  {
  	
  	for(int i=0;i<SIZE;++i){
  		ensure("push_front succeeds",queue.push_front(i));
  	}
  	
	IntQueue::iterator item(queue.begin());
	
	int i=SIZE;
	for(item;item != queue.end();++item){
		ensure("iterate values",*item == --i);
	}
	
  }
}
