#include <tut.h>
#include "util/container/DlList.h"

namespace tut
{
  // Data used by each test
  struct dllist_basic
  { 
	typedef DlList<int> IntList;
    IntList list;
  };

  // Test group registration
  typedef test_group<dllist_basic> factory;
  typedef factory::object object;
}

namespace
{
  tut::factory tf("dllist basic operations");
}

namespace tut
{
   /**
   * Checks empty operation
   */
  template<>
  template<>
  void object::test<1>()
  {
    ensure("empty list",list.empty());
  }
  
  /**
   * Checks push_front operation
   */
  template<>
  template<>
  void object::test<2>()
  {
   
    IntList::node_t node(100);
    list.push_front(&node);
    ensure("list not empty",!list.empty());
    ensure("list.front==100",list.front()->value()==100);

  }
  
  /**
   * Checks pop_front operation
   */
  template<>
  template<>
  void object::test<3>()
  {
  	IntList::node_t node(100);
	list.push_front(&node);
	IntList::node_t* node2=list.pop_front();
    ensure("list empty",list.empty());
    ensure("pop_front",node2->value() == node.value());
  }
  
  /**
   * Check iterator increment operation
   */
  template<>
  template<>
  void object::test<4>()
  {
  	IntList::node_t n1(100);
  	IntList::node_t n2(200);
  	list.push_front(&n2);
  	list.push_front(&n1);
  	
	IntList::iterator item(list.begin());
	++item;
	ensure("increment iterator",item == n2.begin());
  }
  
  /**
   * Check iterator dereference operation
   */
  template<>
  template<>
  void object::test<5>()
  {
  	int nr=5;
  	for(int i=0;i<nr;++i){
  		IntList::node_t* node=new IntList::node_t(i);
  		list.push_front(node);
  	}
	IntList::iterator item(list.begin());
	
	int i=nr;
	for(item;item != list.end();++item){
		ensure("iterate values",*item == --i);
	}
	
  }
  
  /**
   * Check iterator decrement operation
   */
  template<>
  template<>
  void object::test<6>()
  {
  	int nr=5;
  	for(int i=0;i<nr;++i){
  		IntList::node_t* node=new IntList::node_t(i);
  		list.push_front(node);
  	}
	IntList::iterator item(list.last());
	
	int i=0;
	for(item;item != list.end();--item){
 		ensure("iterate values",*item == i++);
	}
  }
  
  
  
}
