#include <tut.h>
#include "util/container/DlQueue.h"

namespace tut
{
  // Data used by each test
  struct dlqueue_basic
  { 
	typedef DlQueue<int> IntQueue;
    IntQueue queue;
  };

  // Test group registration
  typedef test_group<dlqueue_basic> factory;
  typedef factory::object object;
}

namespace
{
  tut::factory tf("dlqueue basic operations");
}

namespace tut
{
   
 /**
   * Checks empty operation
   */
  template<>
  template<>
  void object::test<1>()
  {
    ensure("empty queue",queue.empty());
    ensure("queue.front==0",queue.front()==0);
    ensure("queue.back==0",queue.back()==0);
  }
  
  /**
   * Checks push_front operation
   */
  template<>
  template<>
  void object::test<2>()
  {
   
    IntQueue::node_t node(100);
    queue.push_front(&node);
    ensure("queue not empty",!queue.empty());
    ensure("queue.front.value ==100",queue.front()->value()==100);
    ensure("queue.back.value ==100",queue.back()->value()==100);

  }
  
  /**
   * Checks pop_front operation
   */
  template<>
  template<>
  void object::test<3>()
  {
  	IntQueue::node_t node(100);
	queue.push_front(&node);
	IntQueue::node_t* node2=queue.pop_front();
    ensure("queue empty",queue.empty());
    ensure("pop_front",node2->value() == node.value());
  }
  
  /**
   * Checks push_back operation
   */
  template<>
  template<>
  void object::test<4>()
  {
   
    IntQueue::node_t node(100);
    queue.push_front(&node);
    ensure("queue not empty",!queue.empty());
    ensure("queue.back.value==100",queue.back()->value()==100);

  }
  
  /**
   * Checks pop_back operation
   */
  template<>
  template<>
  void object::test<5>()
  {
  	IntQueue::node_t node(100);
	queue.push_front(&node);
	IntQueue::node_t* node2=queue.pop_front();
    ensure("queue empty",queue.empty());
    ensure("pop_back",node2->value() == node.value());
  }
  
  
  
  
  /**
   * Check iterator increment operation
   */
  template<>
  template<>
  void object::test<6>()
  {
  	IntQueue::node_t n1(100);
  	IntQueue::node_t n2(200);
  	queue.push_front(&n2);
  	queue.push_front(&n1);
  	
	IntQueue::iterator item(queue.begin());
	++item;
	ensure("increment iterator",item == n2.begin());
	ensure("last iterator value",*(queue.last()) == 200);
  }
  
  /**
   * Check iterator dereference operation
   */
  template<>
  template<>
  void object::test<7>()
  {
  	int nr=5;
  	for(int i=0;i<nr;++i){
  		IntQueue::node_t* node=new IntQueue::node_t(i);
  		queue.push_front(node);
  	}
	IntQueue::iterator item(queue.begin());
	
	int i=nr;
	for(item;item != queue.end();++item){
		ensure("iterate values",*item == --i);
	}
	
  } 
  
  /**
   * Check iterator decrement operation
   */
  template<>
  template<>
  void object::test<8>()
  {
  	int nr=5;
  	for(int i=0;i<nr;++i){
  		IntQueue::node_t* node=new IntQueue::node_t(i);
  		queue.push_front(node);
  	}
	IntQueue::iterator item(queue.last());
	
	int i=0;
	for(item;item != queue.end();--item){
 		ensure("iterate values",*item == i++);
	}
  }
  
  
  
}
