<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet 
	xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
	xmlns:str="http://exslt.org/strings"
    xmlns:func="http://exslt.org/functions"
    extension-element-prefixes="str func"
	version="1.0">
	
	<xsl:output method="text"  encoding="iso-8859-1"/>
	<xsl:template match="text()|@*"/>



    <!-- *********************************************************************
		 object_getappid(name)
		 =====================================================================
		 returns the application id of the given task or isr *name*
         ***************************************************************** -->
	
	<xsl:template name="object_getappid">
		<xsl:param name="name"/>
		<xsl:for-each select="//*/osapplication">
			<xsl:variable name="is_theapp">
				<xsl:for-each select="componentref">
					<xsl:call-template name="object_in_component">
						<xsl:with-param name="cname" select="@name"/>
						<xsl:with-param name="oname" select="$name"/>
					</xsl:call-template>
				</xsl:for-each>
			</xsl:variable>
			<xsl:if test="$is_theapp='yes'">
				<xsl:value-of select="@name"/>
			</xsl:if>
		</xsl:for-each>
	</xsl:template>	



    <!-- *********************************************************************
		 object_getcomponent(name)
		 =====================================================================
		 returns the component's name of the given task or isr *name*
         ***************************************************************** -->
	
	<xsl:template name="object_getcomponent">
		<xsl:param name="name"/>
		<xsl:for-each select="//*/component">
			<xsl:variable name="is_thecomponent">		
				<xsl:call-template name="object_in_component">
					<xsl:with-param name="cname" select="@name"/>
					<xsl:with-param name="oname" select="$name"/>
				</xsl:call-template>
			</xsl:variable>
			<xsl:if test="$is_thecomponent='yes'">
				<xsl:value-of select="@name"/>
			</xsl:if>			
		</xsl:for-each>				
	</xsl:template>	



    <!-- *********************************************************************
		 object_in_component(cname,oname)
		 =====================================================================
		 returns "yes" (no text) if the given task or isr *oname* is in
		 	component *cname*
         ***************************************************************** -->

	<xsl:template name="object_in_component">
		<xsl:param name="cname"/>
		<xsl:param name="oname"/>
		<xsl:for-each select="//*/component[@name=$cname]">		
			<xsl:for-each select="taskref|isrref">
				<xsl:if test="@name=$oname">yes</xsl:if>
			</xsl:for-each>
		</xsl:for-each>	
	</xsl:template>



    <!-- *********************************************************************
		 task_funcname(name)
		 isr_funcname(name)
		 =====================================================================
		 return the start function of the given task or isr *name*
         ***************************************************************** -->

	<xsl:template name="task_funcname">
		<xsl:param name="name"/>
		<xsl:for-each select="//*/task[@name=$name]">
			<xsl:call-template name="component_classname">
				<xsl:with-param name="name" select="//*/component/taskref[@name=$name]/../@name"/>
			</xsl:call-template>
			<xsl:text>::</xsl:text>
			<xsl:choose>
				<xsl:when test="@function">
					<xsl:value-of select="@function"/>
				</xsl:when>
				<xsl:otherwise>
					<xsl:text>functionTask</xsl:text>
					<xsl:value-of select="@name"/>
				</xsl:otherwise>
			</xsl:choose>
		</xsl:for-each>
	</xsl:template>	

	<xsl:template name="isr_funcname">
		<xsl:param name="name"/>
		<xsl:for-each select="//*/isr[@name=$name]">
			<xsl:call-template name="component_classname">
				<xsl:with-param name="name" select="//*/component/isrref[@name=$name]/../@name"/>
			</xsl:call-template>
			<xsl:text>::</xsl:text>
			<xsl:choose>
				<xsl:when test="@function">
					<xsl:value-of select="@function"/>
				</xsl:when>
				<xsl:otherwise>
					<xsl:text>functionISR</xsl:text>
					<xsl:value-of select="@name"/>
				</xsl:otherwise>
			</xsl:choose>
		</xsl:for-each>
	</xsl:template>	



    <!-- *********************************************************************
		 component_classname(name)
		 =====================================================================
		 returns the classname belonging to the component *name*
         ***************************************************************** -->

	<xsl:template name="component_classname">
		<xsl:param name="name"/>
		<xsl:for-each select="//*/component[@name=$name]">
			<xsl:choose>
				<xsl:when test="@classname!=''">
					<xsl:value-of select="@classname"/>
				</xsl:when>
				<xsl:otherwise>
					<xsl:value-of select="$name"/>
				</xsl:otherwise>
			</xsl:choose>
		</xsl:for-each>
	</xsl:template>


    <!-- *********************************************************************
		 namespace_mangle(namespace)
		 =====================================================================
		 returns the g++-style mangled prefix of the *namespace*
         ***************************************************************** -->

	<xsl:template name="namespace_mangle">
        <xsl:param name="namespace"/>

		<xsl:choose>
    		<xsl:when test="contains($namespace,'::')">
				<xsl:value-of select="string-length(substring-before($namespace,'::'))"/>
				<xsl:value-of select="substring-before($namespace,':')"/>
				<xsl:call-template name="namespace_mangle">
					<xsl:with-param name="namespace" select="substring-after($namespace,'::')"/>
				</xsl:call-template>
			</xsl:when>
			<xsl:otherwise>
				<xsl:value-of select="string-length($namespace)"/>
				<xsl:value-of select="$namespace"/>
			</xsl:otherwise>
	   	</xsl:choose>
	</xsl:template>

	<xsl:template name="forwarddecl_in_namespace">
        <xsl:param name="classname"/>

		<xsl:choose>
    		<xsl:when test="contains($classname,'::')">
    			<xsl:text>namespace </xsl:text>
				<xsl:value-of select="substring-before($classname,':')"/>
				<xsl:text> {&#10;</xsl:text>
				<xsl:call-template name="forwarddecl_in_namespace">
					<xsl:with-param name="classname" select="substring-after($classname,'::')"/>
				</xsl:call-template>
				<xsl:text>}</xsl:text>				
			</xsl:when>
			<xsl:otherwise>
				<xsl:text>class </xsl:text>
				<xsl:value-of select="$classname"/>
				<xsl:text>;&#10;</xsl:text>								
			</xsl:otherwise>
	   	</xsl:choose>
	</xsl:template>

</xsl:stylesheet>
