#!/usr/bin/perl

use warnings;
use strict;

use SystemConfiguration;

if(@ARGV != 3 || $ARGV[0] !~ /\.xml$/ || $ARGV[2] !~ /\.h$/) {
	print STDERR "Usage: $0 <config.xml> <os.config> <cfAppAttribs.h>\n";
	exit 1;
}

my $cfg = SystemConfiguration->new($ARGV[0], $ARGV[1]);

## Here comes the actual generation code

my %priorityMap = ( );
my $numTasks = 0;
my $maxNumberOfTasksPerPrio;

foreach my $task ($cfg->getTasks()) {
	$numTasks++;

	if(exists $priorityMap{$task->priority()}) {
		$priorityMap{$task->priority()} += 1;
	} else {
		$priorityMap{$task->priority()} = 1;
	}
}

($maxNumberOfTasksPerPrio) = reverse sort { $a <=> $b } values %priorityMap;

my $numCounters = $cfg->getCounters();
my $numAlarms = $cfg->getAlarms();
my $numResources = $cfg->getResources();
my $numISR2 = grep { $_->category() == 2 } $cfg->getISRs();
my $numAppmodes = $cfg->getAppModes();
my @apps = $cfg->getApplications();
my $numApplications = @apps;


open (SRC, ">$ARGV[2]") || die "cannot open $ARGV[2]\n";
print SRC <<EOH;
// generated CiAO runtime
#ifndef CFAPPATTRIBS_H_
#define CFAPPATTRIBS_H_

#define cfOS_NUMBER_OF_TASKS $numTasks
#define cfOS_MAX_NUMBER_OF_TASKS_PER_PRIORITY $maxNumberOfTasksPerPrio

#define cfOS_NUMBER_OF_COUNTERS $numCounters
#define cfOS_NUMBER_OF_ALARMS $numAlarms

#define cfOS_NUMBER_OF_CATEGORY_2_ISRS $numISR2

#define cfOS_NUMBER_OF_APPLICATION_MODES $numAppmodes

#define cfOS_NUMBER_OF_OSAPPLICATIONS $numApplications

EOH

print SRC "#define cfOS_NUMBER_OF_RESOURCES $numResources\n" if $cfg->hasOSFeature('kernel_resources');

print SRC "#endif\n";

close(SRC);
