<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
version="1.0">
<xsl:import href="../ComStacksCommon.xsl"/>
<xsl:output method="text"/>

<!-- Template for all ComStacks -->
<xsl:template match="/ComStacks">
#include "can_Cfg.h"
#include "canif_Cfg.h"
#include "node.h"
#include "hw/hal/can/ControllerConfigType.h"

<xsl:apply-templates />
</xsl:template>


<!-- Template for one node -->
<xsl:template match="/ComStacks/Node">
#if defined(OS_COM_NODE) &#38;&#38; (OS_COM_NODE == <xsl:value-of select="@Name"/>)

<xsl:for-each select="Can/Module">
#include "hw/dev/can/<xsl:value-of select="@Name"/>/CanDriverImpl.h"
</xsl:for-each>

<xsl:if test="count(CanIf/TxLpdu) > 0">
#include "hw/hal/can/tx/TxLpduConfigType.h"
#include "hw/hal/can/tx/TxLpduIdType.h"</xsl:if>

<xsl:if test="count(CanIf/RxLpdu) > 0">
#include "hw/hal/can/rx/RxLpduConfigType.h"
#include "hw/hal/can/rx/RxLpduIdType.h"
#include "hw/hal/can/rx/CanDriverConfigType.h"
#include "hw/hal/can/rx/RxHwObjectConfigType.h"</xsl:if>

namespace hw { namespace hal { namespace can {
<xsl:apply-templates select="CanIf"/>
}}}

#endif // OS_COM_NODE == <xsl:value-of select="@Name"/>
</xsl:template>


<!-- Template for the Can Interface on a node -->
<xsl:template match="/ComStacks/Node/CanIf">
static ControllerConfigType const Controllers[] = {
<xsl:apply-templates select="LogicalController"/>};

<xsl:if test="count(TxLpdu) > 0">
<!-- Create the TxLpdus, currently we assume that lpduids don't have holes and begin with 0 -->
static TxLpduConfigType const TxLpdus[] = {
<xsl:apply-templates select="TxLpdu">
	<xsl:sort select="@LPduId" order="ascending" data-type="number"/>
</xsl:apply-templates>};
</xsl:if>

<xsl:if test="count(RxLpdu) > 0">

static RxLpduConfigType const RxLpdus[] = {
<xsl:apply-templates select="RxLpdu">
	<xsl:sort select="@LPduId" order="ascending" data-type="number"/>	
</xsl:apply-templates>};


<xsl:for-each select="../Can/Module[(count(RxHwObject) > 0) and @Name = ../../CanIf/RxLpdu/@ModuleRef]">
namespace <xsl:value-of select="@Name"/> {

<xsl:for-each select="RxHwObject[@ObjId = ../../../CanIf/RxLpdu[@ModuleRef = current()/@Name]/@HwObjectRef]">
<xsl:text>static RxLpduIdStorageType const RxLpduEntries</xsl:text>
<xsl:value-of select="@ObjId"/><xsl:text>[] = {</xsl:text>
<xsl:for-each select="../../../CanIf/RxLpdu[@HwObjectRef = current()/@ObjId and @ModuleRef = current()/../@Name]">
<xsl:sort select="StandardCanId/@Id | ExtendedCanId/@Id" order="ascending" data-type="number" />
<xsl:value-of select="@LPduId" />
<xsl:if test="position() != last()">, </xsl:if>
</xsl:for-each>};
</xsl:for-each>

static RxHwObjectConfigType const RxHwObjectConfigs[] = {
<xsl:for-each select="TxHwObject | RxHwObject">
<xsl:sort select="@ObjId" order="ascending" data-type="number" /><xsl:text>	{</xsl:text>
<xsl:choose>
<xsl:when test="name() = 'RxHwObject' and (@ObjId = ../../../CanIf/RxLpdu[@ModuleRef = current()/../@Name]/@HwObjectRef)">
	<xsl:text>RxLpduEntries</xsl:text><xsl:value-of select="@ObjId" />, <xsl:value-of select="count(../../../CanIf/RxLpdu[@HwObjectRef = current()/@ObjId and @ModuleRef = current()/../@Name])" /></xsl:when>
<xsl:otherwise><xsl:text>NULL, 0</xsl:text></xsl:otherwise>
</xsl:choose>}<xsl:if test="position() != last()">,
</xsl:if>
</xsl:for-each>
};

}; // namespace <xsl:value-of select="@Name"/><xsl:text>
</xsl:text>
</xsl:for-each>

static CanDriverConfigType const CanDriverConfigs[] = {
<xsl:for-each select="../Can/Module[(count(RxHwObject) > 0) and @Name = ../../CanIf/RxLpdu/@ModuleRef]" >
<xsl:text>	HW_HAL_CAN_DRIVER_CONFIG_TYPE_INITIALIZER((void const *)&#38;hw::dev::can::</xsl:text><xsl:value-of select="@Name" />::CanDriverImpl::theMultican_, <xsl:value-of select="@Name" />::RxHwObjectConfigs, <xsl:value-of select="count(RxHwObject | TxHwObject)" />
<xsl:text>)</xsl:text><xsl:if test="position() != last()">,
</xsl:if>
</xsl:for-each> 
};

</xsl:if> <!-- test="count(RxLpdu) > 0" -->

ConfigType const DefaultConfig = 
	HW_HAL_CAN_CONFIG_TYPE_INITIALIZER(						\
		Controllers, cfHW_HAL_CANIF_NUM_USED_CONTROLLERS,		\
		<xsl:if test="count(TxLpdu) > 0">TxLpdus</xsl:if><xsl:if test="count(TxLpdu) = 0">NULL</xsl:if>, cfHW_HAL_CANIF_NUM_USED_TX_LPDUS,				\
		<xsl:if test="count(RxLpdu) > 0">RxLpdus</xsl:if><xsl:if test="count(RxLpdu) = 0">NULL</xsl:if>, cfHW_HAL_CANIF_NUM_USED_RX_LPDUS,				\
		CanDriverConfigs, <xsl:value-of select="count(../Can/Module[count(RxHwObject[@ObjId = ../../../CanIf/RxLpdu/@HwObjectRef]) > 0])" />);

</xsl:template>


<!-- Template or one TxLpdu -->
<xsl:template match="/ComStacks/Node/CanIf/TxLpdu">
	<xsl:text>	HW_HAL_CAN_TX_LPDU_CONFIG_INITIALIZER((void const *)&#38;hw::dev::can::</xsl:text><xsl:value-of select="@ModuleRef"/><xsl:text>::CanDriverImpl::theMultican_, </xsl:text>
	<xsl:choose>
		<xsl:when test="ExtendedCanId">
			<xsl:value-of select="ExtendedCanId/@Id" /> <xsl:text> + (1&#60;&#60;(sizeof(hw::dev::can::IdType)*8-1)), </xsl:text>
		</xsl:when>
		<xsl:when test="StandardCanId">
			<xsl:value-of select="StandardCanId/@Id" /> <xsl:text>, </xsl:text>
		</xsl:when>
	</xsl:choose>
	<xsl:value-of select="@HwObjectRef"/><xsl:text>)</xsl:text>
	<xsl:if test="position() != last()">,</xsl:if>
	<xsl:text>
</xsl:text>
</xsl:template>


<!-- Template or one RxLpdu-->
<xsl:template match="/ComStacks/Node/CanIf/RxLpdu">
	<xsl:text>	HW_HAL_CAN_RX_LPDU_CONFIG_INITIALIZER(</xsl:text> <!--<xsl:value-of select="@LPduId"/><xsl:text>, </xsl:text> -->
	<xsl:choose>
		<xsl:when test="ExtendedCanId">
			<xsl:value-of select="ExtendedCanId/@Id" /> <xsl:text> + (1&#60;&#60;(sizeof(hw::dev::can::IdType)*8-1)), </xsl:text>
			<xsl:choose>
				<xsl:when test="ExtendedCanId/@Dlc"><xsl:value-of select="ExtendedCanId/@Dlc"/></xsl:when>
				<xsl:otherwise>DLC_UNBOUND</xsl:otherwise>
			</xsl:choose>
		</xsl:when>
		<xsl:when test="StandardCanId">
			<xsl:value-of select="StandardCanId/@Id" /> <xsl:text>, </xsl:text>
			<xsl:choose>
				<xsl:when test="StandardCanId/@Dlc"><xsl:value-of select="StandardCanId/@Dlc"/></xsl:when>
				<xsl:otherwise>DLC_UNBOUND</xsl:otherwise>
			</xsl:choose>
		</xsl:when>
	</xsl:choose>
	<xsl:if test="position() = last()">) 
</xsl:if>
	<xsl:if test="position() != last()">),
</xsl:if>
</xsl:template>

<!-- Template for one LogicalController -->
<xsl:template match="/ComStacks/Node/CanIf/LogicalController">
	<xsl:text>	HW_HAL_CAN_CONTROLLER_CONFIG_INITIALIZER(hw::dev::can::</xsl:text><xsl:value-of select="@ModuleRef"/>
	<xsl:text>::ControllerId</xsl:text><xsl:value-of select="@ControllerRef"/>_Configs, <xsl:value-of select="@ControllerRef"/>
	<xsl:text>, (void const *)&#38;hw::dev::can::</xsl:text><xsl:value-of select="@ModuleRef"/><xsl:text>::CanDriverImpl::theMultican_)</xsl:text> 
	<xsl:if test="position() != last()">,</xsl:if>
	<xsl:text>
</xsl:text>
</xsl:template>

</xsl:stylesheet>

