#!/usr/bin/perl

# Generates messageport_objects.cpp and messageport_objects.h from config.xml
#
# CAVEAT: the make system does not know that this script generates messageport_objects.h,
#   so it is not removed on make clean

use warnings;
use strict;

use SystemConfiguration;

if(@ARGV != 3 || $ARGV[0] !~ /\.xml$/ || $ARGV[2] !~ /\.cpp$/) {
	print STDERR "Usage: $0 <config.xml> <os.config> <messageport_objects.cpp>\n";
	exit 1;
}

my $cfg = SystemConfiguration->new($ARGV[0], $ARGV[1]);

## Here comes the actual generation code
my $headerfile = $ARGV[2];
$headerfile =~ s/\.cpp$/\.h/;
open (SRC, ">$ARGV[2]") || die "cannot open $ARGV[2]\n";
open (HEADER, ">$headerfile") || die "cannot open $headerfile\n";

print HEADER "#ifndef MESSAGEPORT_OBJ_H\n";
print HEADER "#define MESSAGEPORT_OBJ_H\n\n";
print HEADER "#include \"os/krn/Messageport.h\"\n";
print HEADER "#include \"os/mp/Messageport_RevokeWriteSender.h\"\n" if $cfg->hasOSFeature('kernel_messageports_senderwriterevocation');
print HEADER "#include \"os/mp/Messageport_EnableWriteReceiver.h\"\n" if $cfg->hasOSFeature('kernel_messageports_receiverwriteaccess');

print SRC "#include \"$headerfile\"\n\n";

foreach my $mp ($cfg->getMessagePorts()) {
	my $mpk = $mp->{name};

	if ( exists $mp->{msgcheaderfile} && $mp->{msgcheaderfile} ne '' ) {
		print HEADER "#include \"$mp->{msgcheaderfile}\"\n\n";
	}

	my $tmpl = $mp->templateParams();

	print HEADER "extern ".$mp->poolType()." ".$mp->poolSymbol().'['.$mp->poolSize()."];\n\n";

	print HEADER "#define ${mpk}_receiver ".$mp->receiverPortSymbol()."\n";
	print HEADER "extern ".$mp->receiverPortType().$tmpl.' '.$mp->receiverPortSymbol().";\n\n";
	print HEADER "namespace os { namespace krn { namespace ReceiverPortTypes {\n";
	print HEADER "\ttypedef ".$mp->receiverPortType().$tmpl." $mpk;\n";
	print HEADER "}}}\n";

	print HEADER "#define ${mpk}_sender ".$mp->senderPortSymbol()."\n";
	print HEADER "extern ".$mp->senderPortType().$tmpl.' '.$mp->senderPortSymbol().";\n";
	print HEADER "namespace os { namespace krn { namespace SenderPortTypes {\n";
	print HEADER "\ttypedef ".$mp->senderPortType().$tmpl." $mpk;\n";
	print HEADER "}}}\n";

	print SRC $mp->poolType()." ".$mp->poolSymbol().'['.$mp->poolSize()."];\n";

	print SRC $mp->senderPortType()."$tmpl ".$mp->senderPortSymbol(). " = { ";
	print SRC $mp->senderPortInit();
	print SRC "};\n";

	print SRC $mp->receiverPortType()."$tmpl ".$mp->receiverPortSymbol(). " = { ";
	print SRC $mp->receiverPortInit();
	print SRC "};\n";
}

print HEADER "\n#endif\n";

close(SRC);
close(HEADER);

